@@ -1,26 +1,49 @@
-// Copyright (c) 1999 by the University of Southern California
-// All rights reserved.
-//
-// Permission to use, copy, modify, and distribute this software and its
-// documentation in source and binary forms for non-commercial purposes
-// and without fee is hereby granted, provided that the above copyright
-// notice appear in all copies and that both the copyright notice and
-// this permission notice appear in supporting documentation. and that
-// any documentation, advertising materials, and other materials related
-// to such distribution and use acknowledge that the software was
-// developed by the University of Southern California, Information
-// Sciences Institute.  The name of the University may not be used to
-// endorse or promote products derived from this software without
-// specific prior written permission.
-//
-// THE UNIVERSITY OF SOUTHERN CALIFORNIA makes no representations about
-// the suitability of this software for any purpose.  THIS SOFTWARE IS
-// PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
-// INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
-// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
-//
-// Other copyrights might apply to parts of this software and are so
-// noted when applicable.
+
+/*
+ * ew.cc
+ * Copyright (C) 1999 by the University of Southern California
+ * $Id: ew.cc,v 1.7 2006/02/21 15:20:18 mahrenho Exp $
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License,
+ * version 2, as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *
+ * The copyright of this module includes the following
+ * linking-with-specific-other-licenses addition:
+ *
+ * In addition, as a special exception, the copyright holders of
+ * this module give you permission to combine (via static or
+ * dynamic linking) this module with free software programs or
+ * libraries that are released under the GNU LGPL and with code
+ * included in the standard release of ns-2 under the Apache 2.0
+ * license or under otherwise-compatible licenses with advertising
+ * requirements (or modified versions of such code, with unchanged
+ * license).  You may copy and distribute such a system following the
+ * terms of the GNU GPL for this module and the licenses of the
+ * other code concerned, provided that you include the source code of
+ * that other code when and as the GNU GPL requires distribution of
+ * source code.
+ *
+ * Note that people who make modified versions of this module
+ * are not obligated to grant this special exception for their
+ * modified versions; it is their choice whether to do so.  The GNU
+ * General Public License gives permission to release a modified
+ * version without this exception; this exception also makes it
+ * possible to release a modified version which carries forward this
+ * exception.
+ *
+ */
+
 //
 // ew.cc (Early warning system)
 //   by Xuan Chen (xuanc@isi.edu), USC/ISI
@@ -70,40 +93,59 @@ double HLF::getLow() {
 // high(t) = alpha * high(t-1) + (1 - alpha) * o(t)
 // low(t) = (1 - alpha) * low(t-1) + alpha * o(t)
 void HLF::update(double input) {
-  high = alpha * high + (1 - alpha) * input;
-  low = (1 - alpha) * low + alpha * input;
+  // Set values to the current observation for the first time.
+  if (high)
+  	high = alpha * high + (1 - alpha) * input;
+  else
+  	high = input;
+  if (low)
+	low = (1 - alpha) * low + alpha * input;
+  else
+  	low = input;
   //  printf("HLF %d %.2f, %.2f\n", (int)now, high, low);
 }
 
-// Definition for a tocken-bucket rate limitor
+// Definition for a token-bucket rate limitor
 TBrateLimitor::TBrateLimitor() {
   TBrateLimitor(DEFAULT_TB_RATE_P);
-};
+}
 
 TBrateLimitor::TBrateLimitor(double rate) {
   double now = Scheduler::instance().clock();
   pkt_mode = 1;
   bucket_size = DEFAULT_TB_SIZE;
-  tocken_rate = 0;
-  tocken_num = bucket_size;
+  token_rate = 0;
+  token_num = bucket_size;
   last_time = now;
-  ini_tocken_rate = rate;
-
-  setRate(rate);
-  //printf("TB pkt_mode:%d, bucket_size:%g, tocken_num:%g, last_time:%g\n",
-  //	 pkt_mode, bucket_size, tocken_num, last_time);
+  ini_token_rate = rate;
 
   resetScore();
-};
+  setRate(rate);
+  //printf("TB pkt_mode:%d, bucket_size:%g, token_num:%g, last_time:%g\n",
+  //	 pkt_mode, bucket_size, token_num, last_time);
+
+  // High-low filter
+  hlf.setAlpha(ALPHA);
+  hlf.reset(rate);
+}
 
 // adjust the rate limit to the default value
 void TBrateLimitor::setRate(double rate) {
-  if (rate != tocken_rate) {
-    last_tocken_rate = tocken_rate;
-    tocken_rate = rate;
-    printf("TR %d %.2f %d %d\n", (int)(Scheduler::instance().clock()), 
-	   tocken_rate, p_score, n_score);
+  last_token_rate = token_rate;
+  
+  if (! token_rate) {
+    token_rate = rate;
+  } else if (rate != token_rate) {
+    // use HLF to change token rate
+    hlf.update(rate);
+    // Use low-gain filter for fast response
+    //token_rate = hlf.getLow();
+    token_rate = hlf.getHigh();
+    token_rate = rate;
   }
+  
+  printf("TR %d %.2f %.2f %d %d\n", (int)(Scheduler::instance().clock()), 
+	 token_rate, rate, p_score, n_score);
 }
 
 // adjust the rate limit to approaching an optimal rate limit
@@ -111,11 +153,11 @@ void TBrateLimitor::adjustRate() {
   // pay the penalty
   adjustScore(-1);
 
-  double rate = tocken_rate;
+  double rate = token_rate;
   if (p_score >= n_score)
-    rate = tocken_rate * (1 + 0.2);
+    rate = token_rate * (1 + 0.2);
   else 
-    rate = tocken_rate * (1 - 0.2);
+    rate = token_rate * (1 - 0.2);
 
   setRate(rate);
 }
@@ -128,7 +170,7 @@ void TBrateLimitor::resetScore() {
 // adjust the score for increasing or decreasing scores
 void TBrateLimitor::adjustScore(int score) {
   // pay the penalty
-  if (last_tocken_rate > tocken_rate)
+  if (last_token_rate > token_rate)
     n_score += score;
   else
     p_score += score;
@@ -139,18 +181,18 @@ int TBrateLimitor::run(double incoming, double t_rate) {
   double interval = now - last_time;
   
   //printf("TB: now:%g last_time:%g interval:%g; ", now, last_time, interval);
-  tocken_num += interval * t_rate;
+  token_num += interval * t_rate;
   last_time = now;
 
-  // more tockens are overflowed
-  if (tocken_num > bucket_size)
-    tocken_num = bucket_size;
+  // more tokens are overflowed
+  if (token_num > bucket_size)
+    token_num = bucket_size;
 
-  //printf("tocken #:%g; ", tocken_num);
+  //printf("token #:%g; ", token_num);
 
   // through (0 dropping probability)
-  if (tocken_num >= incoming) {
-    tocken_num -= incoming;
+  if (token_num >= incoming) {
+    token_num -= incoming;
     //printf("...through\n");
     return 0;
   }
@@ -161,7 +203,7 @@ int TBrateLimitor::run(double incoming, double t_rate) {
 }
 
 int TBrateLimitor::run(double incoming) {
-  return (run(incoming, tocken_rate));
+  return (run(incoming, token_rate));
 }
 
 // EW detector
@@ -180,7 +222,7 @@ EWdetector::EWdetector() {
   resetChange();
 
   // High-low filter
-  hlf.setAlpha(EW_FADING_FACTOR);
+  hlf.setAlpha(ALPHA);
 }
 
 //EWdetector::~EWdetector() {
@@ -190,17 +232,18 @@ EWdetector::EWdetector() {
 void EWdetector::setDt(int inv) {
   dt_inv = inv;
   //printf("DT: %d\n", dt_inv);
-};
+}
+
 void EWdetector::setDb(int inv) {
   db_inv = inv;
   //printf("DB: %d\n", db_inv);
-};
+}
 
 void EWdetector::setLink(int src, int dst) {
   ew_src = src;
   ew_dst = dst;
   //printf("EW: (%d:%d)\n", ew_src, ew_dst);
-};
+}
 
 void EWdetector::setAlarm() {
   alarm = 1;
@@ -208,7 +251,7 @@ void EWdetector::setAlarm() {
   // Reset low and high gain filters' input values to the long-term avg
   // Actually, there is no change to high gain filter
   hlf.reset(avg_rate);
-};
+}
 
 void EWdetector::resetAlarm() {
   alarm = 0;
@@ -216,7 +259,7 @@ void EWdetector::resetAlarm() {
   // Reset low and high gain filters' input values to the long-term avg
   // Actually, there is no change to low gain filter
   hlf.reset(avg_rate);
-};
+}
 
 // Set and reset change flag
 void EWdetector::setChange() {
@@ -250,7 +293,8 @@ void EWdetector::updateAvg() {
   // Update the long term average value with the output from different filters
   if (!alarm) {
     // Use low-gain filter for fast response
-    avg_rate = hlf.getLow();
+    //avg_rate = hlf.getLow();
+    avg_rate = hlf.getHigh();
   } else {
     // Use high-gain filter to keep the long term average stable
     avg_rate = hlf.getHigh();
@@ -707,14 +751,14 @@ void EWdetectorB::detect() {
   //  detect if it is the time to trigger the alarm
   if (alarm) {
     // Determine if an alarm should be released
-    if (cur_rate > avg_rate * (1 + EW_DETECT_RANGE)) {
+    if (cur_rate > avg_rate * (1 + EW_RELEASE_RANGE)) {
       // reset alarm
       resetAlarm();
     } 
   } else {
     // Determine if an alarm should be triggered
     //   need to be conservative!
-    if (cur_rate < avg_rate) {
+    if (cur_rate < avg_rate * (1 - EW_DETECT_RANGE)) {
       setAlarm();
       
       // Initial drop_p to the MAX value whenever alarm triggered
@@ -747,7 +791,7 @@ void EWdetectorB::computeDropP() {
       p = 0;
     
     // Compute the actual drop probability
-    drop_p = EW_FADING_FACTOR * drop_p + (1 - EW_FADING_FACTOR) * p;    
+    drop_p = ALPHA * drop_p + (1 - ALPHA) * p;    
     // adjust drop_p
     if (drop_p < EW_MIN_DROP_P)
       drop_p = EW_MIN_DROP_P;
@@ -759,7 +803,7 @@ void EWdetectorB::computeDropP() {
       if (drop_p <= EW_MIN_DROP_P)
         drop_p = 0;
       else {
-        drop_p = EW_FADING_FACTOR * drop_p;
+        drop_p = ALPHA * drop_p;
       }
     }
   }
@@ -893,7 +937,7 @@ void EWdetectorP::detect() {
     if (!alarm) {
       setAlarm();
     }
-  } else if (cur_rate < avg_rate * (1 - EW_DETECT_RANGE)) {
+  } else if (cur_rate < avg_rate * (1 - EW_RELEASE_RANGE)) {
     if (alarm)
       resetAlarm();
   }
@@ -991,6 +1035,10 @@ int EWPolicy::applyPolicer(policyTableEntry *policy, policerTableEntry *policer,
   //   with req:  may cause resp retransmission.
   // just pass them through
   hdr_cmn *th = hdr_cmn::access(pkt);
+  hdr_ip* iph = hdr_ip::access(pkt);
+  int dst_id = iph->daddr();
+  int src_id = iph->saddr();
+
   if (th->ptype() == PT_ACK)
     return(policer->initialCodePt);
 
@@ -1000,8 +1048,10 @@ int EWPolicy::applyPolicer(policyTableEntry *policy, policerTableEntry *policer,
   now = Scheduler::instance().clock();
 
   // keep arrival packet stats
-  if (ewP)
+  if (ewP) {
+    printf("TRR %d %d %d %d\n", (int)now, src_id, dst_id, th->size());
     ewP->updateStats(PKT_ARRIVAL);
+  }
 
   // For other packets:
   if (dropPacket(pkt)) {
@@ -1166,23 +1216,23 @@ void EWPolicy::detectBr() {
 void EWPolicy::limitPr(double rate) {
   //printf("PR %d\n", rate);
   rlP = new TBrateLimitor(rate);
-};
+}
 
 // Rate limitor: bit rate
 void EWPolicy::limitBr(double rate) {
   //printf("BR %d\n", rate);
   rlB = new TBrateLimitor(rate);
-};
+}
 
 // Rate limitor: packet rate
 void EWPolicy::limitPr() {
   limitPr(DEFAULT_TB_RATE_P);
-};
+}
 
 // Rate limitor: bit rate
 void EWPolicy::limitBr() {
   limitBr(DEFAULT_TB_RATE_B);
-};
+}
 
 // couple EW detector
 void EWPolicy::coupleEW(EWPolicy *ewpc) {
