@@ -1,23 +1,48 @@
-// Copyright (c) 2000 by the University of Southern California
-// All rights reserved.
-//
-// Permission to use, copy, modify, and distribute this software and its
-// documentation in source and binary forms for non-commercial purposes
-// and without fee is hereby granted, provided that the above copyright
-// notice appear in all copies and that both the copyright notice and
-// this permission notice appear in supporting documentation. and that
-// any documentation, advertising materials, and other materials related
-// to such distribution and use acknowledge that the software was
-// developed by the University of Southern California, Information
-// Sciences Institute.  The name of the University may not be used to
-// endorse or promote products derived from this software without
-// specific prior written permission.
-//
-// THE UNIVERSITY OF SOUTHERN CALIFORNIA makes no representations about
-// the suitability of this software for any purpose.  THIS SOFTWARE IS
-// PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
-// INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
-// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+
+/*
+ * dsragent.cc
+ * Copyright (C) 2000 by the University of Southern California
+ * $Id: dsragent.cc,v 1.37 2006/03/10 12:25:28 mahrenho Exp $
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License,
+ * version 2, as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *
+ * The copyright of this module includes the following
+ * linking-with-specific-other-licenses addition:
+ *
+ * In addition, as a special exception, the copyright holders of
+ * this module give you permission to combine (via static or
+ * dynamic linking) this module with free software programs or
+ * libraries that are released under the GNU LGPL and with code
+ * included in the standard release of ns-2 under the Apache 2.0
+ * license or under otherwise-compatible licenses with advertising
+ * requirements (or modified versions of such code, with unchanged
+ * license).  You may copy and distribute such a system following the
+ * terms of the GNU GPL for this module and the licenses of the
+ * other code concerned, provided that you include the source code of
+ * that other code when and as the GNU GPL requires distribution of
+ * source code.
+ *
+ * Note that people who make modified versions of this module
+ * are not obligated to grant this special exception for their
+ * modified versions; it is their choice whether to do so.  The GNU
+ * General Public License gives permission to release a modified
+ * version without this exception; this exception also makes it
+ * possible to release a modified version which carries forward this
+ * exception.
+ *
+ */
 //
 // Other copyrights might apply to parts of this software and are so
 // noted when applicable.
@@ -56,7 +81,6 @@ extern "C" {
 #include "requesttable.h"
 #include "dsragent.h"
 
-
 /*==============================================================
   Declarations and global defintions
 ------------------------------------------------------------*/
@@ -100,6 +124,7 @@ DSRAgent_List DSRAgent::agthead = { 0 };
 Time arp_timeout = 30.0e-3;	// (sec) arp request timeout
 Time rt_rq_period = 0.5;	// (sec) length of one backoff period
 Time rt_rq_max_period = 10.0;	// (sec) maximum time between rt reqs
+Time send_timeout = SEND_TIMEOUT; // (sec) how long a packet can live in sendbuf
 
 #if 0
 /* used in route reply holdoffs, which are currently disabled -dam 5/98 */
@@ -188,6 +213,22 @@ Our strategy is as follows:
 
 */
 
+/* Callback helpers */
+void
+XmitFailureCallback(Packet *pkt, void *data)
+{
+  DSRAgent *agent = (DSRAgent *)data; // cast of trust
+  agent->xmitFailed(pkt);
+}
+
+void
+XmitFlowFailureCallback(Packet *pkt, void *data)
+{
+  DSRAgent *agent = (DSRAgent *)data;
+  agent->xmitFlowFailed(pkt);
+}
+
+
 /*===========================================================================
   SendBuf management and helpers
 ---------------------------------------------------------------------------*/
@@ -252,7 +293,7 @@ DSRAgent::sendBufferCheck()
   for (c  = 0 ; c <SEND_BUF_SIZE ; c++) {
 	  if (send_buf[c].p.pkt == NULL)
 		  continue;
-	  if (Scheduler::instance().clock() - send_buf[c].t > SEND_TIMEOUT) {
+	  if (Scheduler::instance().clock() - send_buf[c].t > send_timeout) {
 		  dropSendBuff(send_buf[c].p);
 		  send_buf[c].p.pkt = 0;
 		  continue;
@@ -465,6 +506,22 @@ DSRAgent::command(int argc, const char*const* argv)
 	  route_cache->MAC_id = MAC_id;
 	  return TCL_OK;
 	}
+      else if(strcasecmp(argv[1], "rt_rq_max_period") == 0)
+        {
+          rt_rq_max_period = strtod(argv[2],NULL);
+          return TCL_OK;
+        }
+      else if(strcasecmp(argv[1], "rt_rq_period") == 0)
+        {
+          rt_rq_period = strtod(argv[2],NULL);
+          return TCL_OK;
+        }
+      else if(strcasecmp(argv[1], "send_timeout") == 0)
+        {
+          send_timeout = strtod(argv[2],NULL);
+          return TCL_OK;
+        }
+
       
       if( (obj = TclObject::lookup(argv[2])) == 0) 
 	{
@@ -640,8 +697,7 @@ DSRAgent::handlePktWithoutSR(SRPacket& p, bool retry)
   /* obtain a source route to p's destination and send it off.
      this should be a retry if the packet is already in the sendbuffer */
 {
-  hdr_sr *srh =  hdr_sr::access(p.pkt);
-  assert(srh->valid());
+  assert(HDR_SR (p.pkt)->valid());
 
   if (p.dest == net_id)
     { // it doesn't need a source route, 'cause it's for us
@@ -2744,19 +2800,6 @@ DSRAgent::xmitFailed(Packet *pkt, const char* reason)
   sendOutPacketWithRoute(p, true);
 }
 
-void
-XmitFailureCallback(Packet *pkt, void *data)
-{
-  DSRAgent *agent = (DSRAgent *)data; // cast of trust
-  agent->xmitFailed(pkt);
-}
-
-void
-XmitFlowFailureCallback(Packet *pkt, void *data)
-{
-  DSRAgent *agent = (DSRAgent *)data;
-  agent->xmitFlowFailed(pkt);
-}
 
 #if 0
 
