@@ -1,21 +1,50 @@
+
 /*
- * Copyright (c) 2001 University of Southern California.
- * All rights reserved.
+ * lms-receiver.cc
+ * Copyright (C) 2001 by the University of Southern California
+ * $Id: lms-receiver.cc,v 1.4 2006/02/21 15:20:19 mahrenho Exp $
  *
- * Redistribution and use in source and binary forms are permitted
- * provided that the above copyright notice and this paragraph are
- * duplicated in all such forms and that any documentation, advertising
- * materials, and other materials related to such distribution and use
- * acknowledge that the software was developed by the University of
- * Southern California, Information Sciences Institute.  The name of the
- * University may not be used to endorse or promote products derived from
- * this software without specific prior written permission.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License,
+ * version 2, as published by the Free Software Foundation.
  *
- * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
- * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
- * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- **
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *
+ * The copyright of this module includes the following
+ * linking-with-specific-other-licenses addition:
+ *
+ * In addition, as a special exception, the copyright holders of
+ * this module give you permission to combine (via static or
+ * dynamic linking) this module with free software programs or
+ * libraries that are released under the GNU LGPL and with code
+ * included in the standard release of ns-2 under the Apache 2.0
+ * license or under otherwise-compatible licenses with advertising
+ * requirements (or modified versions of such code, with unchanged
+ * license).  You may copy and distribute such a system following the
+ * terms of the GNU GPL for this module and the licenses of the
+ * other code concerned, provided that you include the source code of
+ * that other code when and as the GNU GPL requires distribution of
+ * source code.
+ *
+ * Note that people who make modified versions of this module
+ * are not obligated to grant this special exception for their
+ * modified versions; it is their choice whether to do so.  The GNU
+ * General Public License gives permission to release a modified
+ * version without this exception; this exception also makes it
+ * possible to release a modified version which carries forward this
+ * exception.
+ *
+ */
+
+/*
  * Light-Weight Multicast Services (LMS), Reliable Multicast
  *
  * lms-receiver.cc
@@ -142,7 +171,7 @@ class LmsNak {
 	int		fid_;
 };
 
-LmsNak::LmsNak(LmsReceiver *a, int lo, int hi): a_(a), lo_(lo), hi_(hi), nak_timer_(this)
+LmsNak::LmsNak(LmsReceiver *a, int lo, int hi): nak_timer_(this), lo_(lo), hi_(hi), a_(a)
 {
 	seqn_ = 0;
 }
@@ -208,7 +237,6 @@ void LmsReceiver::send_nak (int sqn, int lo, int hi)
 	ch->size_ = sizeof(struct lms_nak) + sizeof(hdr_lms); 
 	iph->daddr() = upstream_lms_;
 	iph->flowid() = 8;
-	double now= Scheduler::instance().clock();
 #ifdef LMS_DEBUG
 printf("at %f %s send nak to upstream lms %d, size is %d\n\n",
   now,uname_, upstream_lms_, (HDR_CMN(p))->size_);
@@ -408,7 +436,6 @@ printf ("%s LMS_SETUP from %d:%d src %d:%d group 0x%x RTT %f dataSize %d\n\n",
 		case LMS_SPM:
 			{
 			struct lms_spm *spm = (struct lms_spm *)pkt->accessdata ();
-			nsaddr_t adr = spm->spm_path_;
 #ifdef LMS_DEBUG    
 printf ("%s LMS_SPM seqno %d, upstream %d:%d\n\n",
   uname_, spm->spm_seqno_, adr>>8, adr&0xff);
@@ -619,17 +646,17 @@ void LmsReceiver::print_all_stats (int drops)
 	//
 	if (drops)
 		{
-                    printf ("\t%.5lf\t  %d\t%.4lf", aveTotalDupReqs, max_dup_naks_,
-                                 ((double)dup_replies/(double)(nrcvs*drops)) );
+                    printf ("\t%.5f\t  %d\t%.4f", aveTotalDupReqs, max_dup_naks_,
+			    ((double)dup_replies/(double)(nrcvs*drops)) );
 		}
 	else	{
 		printf ("\t0.0\t  0\t0.0");
 		}
 
-	//printf ("\t%d/%d/%.2lf", nrcvs, cnt1, (cnt1)?((double)dup_replies/(double)cnt1):0.0);
+	//printf ("\t%d/%d/%.2f", nrcvs, cnt1, (cnt1)?((double)dup_replies/(double)cnt1):0.0);
 	if (cnt2) 
 		{
-		printf ("\t%.2lf\t%.2lf\t%.2lf\n", avg_norm_latency/(double)cnt2, min, max);
+		printf ("\t%.2f\t%.2f\t%.2f\n", avg_norm_latency/(double)cnt2, min, max);
 		}
 	else
 		{
@@ -651,13 +678,13 @@ void LmsReceiver::print_stats ()
         if (rcvbuf_->pkts_recovered_)
                 {
                 printf ("\tPackets recovered:\t%d\n", rcvbuf_->pkts_recovered_);
-                printf ("\tNormalized latency (min, max, avg):\t%lf, %lf, %lf\n",
+                printf ("\tNormalized latency (min, max, avg):\t%f, %f, %f\n",
                         rcvbuf_->min_delay_/rtt_, rcvbuf_->max_delay_/rtt_,
                         (rcvbuf_->delay_sum_/(double)rcvbuf_->pkts_recovered_)/rtt_);
                 }
         printf ("\tDuplicate Replies:\t%d\n", rcvbuf_->duplicates_);
         printf ("\tDuplicate Requests:\t%d\n", dup_reqs_);
-        printf ("\tRtt:\t\t\t%lf\n", rtt_);
+        printf ("\tRtt:\t\t\t%f\n", rtt_);
         printf ("\n");
 }
 
@@ -696,6 +723,7 @@ LmsNak *LmsReceiver::find_nak (int i)
 	// NOT REACHED
 	printf("ERROR: %s nak %d not found\n", uname_, i);
 	abort ();
+	return 0;
 }
 
 void LmsReceiver::delete_nak (LmsNak *n)
