@@ -1,34 +1,54 @@
 // -*-	Mode:C++; c-basic-offset:8; tab-width:8; indent-tabs-mode:t -*-
-//
-// Time-stamp: <2000-09-11 15:24:28 haoboy>
-//
-// Copyright (c) 2000 by the University of Southern California
-// All rights reserved.
-//
-// Permission to use, copy, modify, and distribute this software and its
-// documentation in source and binary forms for non-commercial purposes
-// and without fee is hereby granted, provided that the above copyright
-// notice appear in all copies and that both the copyright notice and
-// this permission notice appear in supporting documentation. and that
-// any documentation, advertising materials, and other materials related
-// to such distribution and use acknowledge that the software was
-// developed by the University of Southern California, Information
-// Sciences Institute.  The name of the University may not be used to
-// endorse or promote products derived from this software without
-// specific prior written permission.
-//
-// THE UNIVERSITY OF SOUTHERN CALIFORNIA makes no representations about
-// the suitability of this software for any purpose.  THIS SOFTWARE IS
-// PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
-// INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
-// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
-//
-// Other copyrights might apply to parts of this software and are so
-// noted when applicable.
+
+/*
+ * classifier-addr-mpls.cc
+ * Copyright (C) 2000 by the University of Southern California
+ * $Id: classifier-addr-mpls.cc,v 1.7 2005/08/25 18:58:09 johnh Exp $
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License,
+ * version 2, as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *
+ * The copyright of this module includes the following
+ * linking-with-specific-other-licenses addition:
+ *
+ * In addition, as a special exception, the copyright holders of
+ * this module give you permission to combine (via static or
+ * dynamic linking) this module with free software programs or
+ * libraries that are released under the GNU LGPL and with code
+ * included in the standard release of ns-2 under the Apache 2.0
+ * license or under otherwise-compatible licenses with advertising
+ * requirements (or modified versions of such code, with unchanged
+ * license).  You may copy and distribute such a system following the
+ * terms of the GNU GPL for this module and the licenses of the
+ * other code concerned, provided that you include the source code of
+ * that other code when and as the GNU GPL requires distribution of
+ * source code.
+ *
+ * Note that people who make modified versions of this module
+ * are not obligated to grant this special exception for their
+ * modified versions; it is their choice whether to do so.  The GNU
+ * General Public License gives permission to release a modified
+ * version without this exception; this exception also makes it
+ * possible to release a modified version which carries forward this
+ * exception.
+ *
+ */
+
 //
 // Original source contributed by Gaeil Ahn. See below.
 //
-// $Header: /nfs/jade/vint/CVSROOT/ns-2/mpls/classifier-addr-mpls.cc,v 1.5 2001/03/06 20:53:41 haldar Exp $
+// $Header: /cvsroot/nsnam/ns-2/mpls/classifier-addr-mpls.cc,v 1.7 2005/08/25 18:58:09 johnh Exp $
 
 // XXX
 //
@@ -628,18 +648,12 @@ int MPLSAddressClassifier::command(int argc, const char*const* argv)
 	} else if (argc == 8) {      
 		int addr  = atoi(argv[2]);
 		int LSPid = atoi(argv[3]);
-		int PHB   = LSPid;
-		int LIBptr, PFTnb,ERBnb ;
-		if (LSPid == MPLS_DEFAULT_PHB)   // topology-based LSP
-			PFTnb = PFTlocate(addr,PHB,LIBptr);
-		else                // ER-LSP
-			ERBnb = ERBlocate(LSPid,addr,LIBptr);
-		
-		if (strcmp(argv[1], "LSPrelease") == 0) {
-			// <classifier> LSPrelease <fec> <lspid> <iif> 
-			// 	<ilabel> <oif> <olabel>
-			if (LSPid < 0) {
-				// Topology-based LSP
+		int LIBptr;
+		if (LSPid == MPLS_DEFAULT_PHB)  {
+			// topology-based LSP
+			int PHB   = LSPid;
+			int PFTnb = PFTlocate(addr,PHB,LIBptr);
+			if (strcmp(argv[1], "LSPrelease") == 0) {
 				if (PFTnb >= 0) {
 					// PFT entry exist
 					LIBupdate(LIBptr, atoi(argv[4]), 
@@ -648,25 +662,8 @@ int MPLSAddressClassifier::command(int argc, const char*const* argv)
 					if (LIBisdeleted(LIBptr) == 0)
 						PFTdeleteLIBptr(LIBptr);
 				}
-			} else {
-				// ER-LSP
-				if ( ERBnb >= 0 ) {
-					// ERB entry exist
-					LIBupdate(LIBptr, atoi(argv[4]),
-						  atoi(argv[5]), atoi(argv[6]),
-						  atoi(argv[7]));
-					if (LIBisdeleted(LIBptr) == 0) {
-						ERBdelete(ERBnb);
-						PFTdeleteLIBptr(LIBptr);
-					}
-				}
-			}
-			return (TCL_OK);        
-		} else if (strcmp(argv[1], "LSPsetup") == 0) {
-			// <classifier> LSPsetup <fec> <lspid> <iif> 
-			//     <ilabel> <oif> <olabel>
-			if (LSPid == MPLS_DEFAULT_PHB) {
-				// Topology-based LSP
+				return (TCL_OK);
+			} else if (strcmp(argv[1], "LSPsetup") == 0) {
 				if (PFTnb < 0) {
 					// PFT entry not exist
 					int ptr = LIBinsert(atoi(argv[4]),
@@ -722,35 +719,50 @@ int MPLSAddressClassifier::command(int argc, const char*const* argv)
 				}
 				return (TCL_OK);
 			}
-			// LSPid != MPLS_DEFAULT_PHB
+		} else {
 			// ER-LSP
-			if (ERBnb < 0) {
-				// ERB entry not exist
-				int ptr = LIBinsert(atoi(argv[4]),
-						    atoi(argv[5]),
-						    atoi(argv[6]),
-						    atoi(argv[7]));
-				if (ptr > -1) {
-					ERBinsert(LSPid,addr,ptr);
-					return (TCL_OK);
-				} else
-					return (TCL_ERROR);
-			} 
-			// ERBnb >= 0
-			// ERB entry already exist
-			if (LIBptr > -1)
-				LIBupdate(LIBptr, atoi(argv[4]), atoi(argv[5]),
-					  atoi(argv[6]), atoi(argv[7]));
-			else {
-				int ptr = 
-					LIBinsert(atoi(argv[4]),atoi(argv[5]),
-						  atoi(argv[6]),atoi(argv[7]));
-				if (ptr > -1)
-					ERBupdate(ERBnb, ptr);
-				else
-					return (TCL_ERROR);
+			int ERBnb = ERBlocate(LSPid,addr,LIBptr);
+			if (strcmp(argv[1], "LSPrelease") == 0) {
+				if ( ERBnb >= 0 ) {
+					// ERB entry exist
+					LIBupdate(LIBptr, atoi(argv[4]),
+						  atoi(argv[5]), atoi(argv[6]),
+						  atoi(argv[7]));
+					if (LIBisdeleted(LIBptr) == 0) {
+						ERBdelete(ERBnb);
+						PFTdeleteLIBptr(LIBptr);
+					}
+				}
+				return (TCL_OK);
+			} else if (strcmp(argv[1], "LSPsetup") == 0) {
+				if (ERBnb < 0) {
+					// ERB entry not exist
+					int ptr = LIBinsert(atoi(argv[4]),
+							    atoi(argv[5]),
+							    atoi(argv[6]),
+							    atoi(argv[7]));
+					if (ptr > -1) {
+						ERBinsert(LSPid,addr,ptr);
+						return (TCL_OK);
+					} else
+						return (TCL_ERROR);
+				} 
+				// ERBnb >= 0
+				// ERB entry already exist
+				if (LIBptr > -1)
+					LIBupdate(LIBptr, atoi(argv[4]), atoi(argv[5]),
+						  atoi(argv[6]), atoi(argv[7]));
+				else {
+					int ptr = 
+						LIBinsert(atoi(argv[4]),atoi(argv[5]),
+							  atoi(argv[6]),atoi(argv[7]));
+					if (ptr > -1)
+						ERBupdate(ERBnb, ptr);
+					else
+						return (TCL_ERROR);
+				}
+				return (TCL_OK);
 			}
-			return (TCL_OK);
 		}
 	}
   
