@@ -1,21 +1,50 @@
+
 /*
- * Copyright (c) 2001 University of Southern California.
- * All rights reserved.                                            
- *                                                                
- * Redistribution and use in source and binary forms are permitted
- * provided that the above copyright notice and this paragraph are
- * duplicated in all such forms and that any documentation, advertising
- * materials, and other materials related to such distribution and use
- * acknowledge that the software was developed by the University of
- * Southern California, Information Sciences Institute.  The name of the
- * University may not be used to endorse or promote products derived from
- * this software without specific prior written permission.
- * 
- * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
- * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
- * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * pgm-sender.cc
+ * Copyright (C) 2001 by the University of Southern California
+ * $Id: pgm-sender.cc,v 1.12 2006/02/21 15:20:19 mahrenho Exp $
  *
- **
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License,
+ * version 2, as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *
+ * The copyright of this module includes the following
+ * linking-with-specific-other-licenses addition:
+ *
+ * In addition, as a special exception, the copyright holders of
+ * this module give you permission to combine (via static or
+ * dynamic linking) this module with free software programs or
+ * libraries that are released under the GNU LGPL and with code
+ * included in the standard release of ns-2 under the Apache 2.0
+ * license or under otherwise-compatible licenses with advertising
+ * requirements (or modified versions of such code, with unchanged
+ * license).  You may copy and distribute such a system following the
+ * terms of the GNU GPL for this module and the licenses of the
+ * other code concerned, provided that you include the source code of
+ * that other code when and as the GNU GPL requires distribution of
+ * source code.
+ *
+ * Note that people who make modified versions of this module
+ * are not obligated to grant this special exception for their
+ * modified versions; it is their choice whether to do so.  The GNU
+ * General Public License gives permission to release a modified
+ * version without this exception; this exception also makes it
+ * possible to release a modified version which carries forward this
+ * exception.
+ *
+ */
+
+/*
  * Pragmatic General Multicast (PGM), Reliable Multicast
  *
  * pgm-sender.cc
@@ -80,7 +109,7 @@ protected:
 class RdataItem {
 public:
   RdataItem(int seqno, PgmSender *a, Packet *rdata_pkt = NULL) :
-    seqno_(seqno), rdata_timer_(a, TIMER_RDATA), rdata_pkt_(rdata_pkt) { }
+    seqno_(seqno), rdata_pkt_(rdata_pkt), rdata_timer_(a, TIMER_RDATA) { }
 
   // Allow direct access to the private data.
   int & seqno() { return seqno_; }
@@ -157,7 +186,7 @@ protected:
 
   void display_packet(Packet *pkt); // For debugging.
 
-  void PgmSender::trace_event(char *evType, nsaddr_t daddr, double evTime); 
+  void trace_event(char *evType, nsaddr_t daddr, double evTime); 
 
   EventTrace * et_; 	//Trace Object for custom Event Traces
 
@@ -385,9 +414,9 @@ void PgmSender::handle_nak(Packet *pkt)
 
   hdr_cmn *hc = HDR_CMN(pkt);
   hdr_pgm *hp = HDR_PGM(pkt);
-  hdr_pgm_nak *pnak = HDR_PGM_NAK(pkt);
+  //hdr_pgm_nak *pnak = HDR_PGM_NAK(pkt);
 
-  if (!(hp->tsi_ == here_)) {
+  if (!(hp->tsi_.isEqual (here_))) {
     printf("%s received NAK with wrong TSI, discarding.\n", uname_);
     return;
   }
@@ -600,33 +629,23 @@ void PgmSender::send_rdata(RdataItem *item)
   stats_.num_rdata_sent_++;
 
   // Send the packet to each of the interfaces.
-  NsObject *tgt;
-  Packet *pkt, *rDataPkt;
-  hdr_ip *rdata_hip; 
-
-  // Used to determine when we need to make additional copies of the packet.
-  int flag = 0;
-
   if (!item->iface_list().empty()) {
     list<int>::iterator iter = item->iface_list().begin();
-
+    int flag = 0;   // Used to determine when we need to make additional copies of the packet.
     while (iter != item->iface_list().end()) {
-      if (!flag) {
-	tgt = iface2link(*iter);
-	tgt->recv(item->rdata_pkt());
-    rDataPkt = item->rdata_pkt();
-    rdata_hip = HDR_IP(pkt);
-    trace_event("SEND RDATA", rdata_hip->daddr(), 0);
-	flag = 1;
-      }
-      else {
+      NsObject *tgt;
+      tgt = iface2link(*iter);
+      Packet *pkt = item->rdata_pkt();
+      if (flag) {
 	// Make a copy of each packet before sending it, so we don't free()
 	// the same packet at the different receivers causing a deallocation
 	// problem.
-	pkt = item->rdata_pkt()->copy();
-	tgt = iface2link(*iter);
-	tgt->recv(pkt);
+	pkt = pkt->copy();
+      } else {
+	trace_event("SEND RDATA", HDR_IP(pkt)->daddr(), 0);
+	flag = 1;
       }
+      tgt->recv(pkt);
 
       iter++;
     }
@@ -635,16 +654,15 @@ void PgmSender::send_rdata(RdataItem *item)
 
   if (!item->agent_list().empty()) {
     list<NsObject *>::iterator iter = item->agent_list().begin();
-
+    int flag = 0;
     while (iter != item->agent_list().end()) {
-      if (!flag) {
-	(*iter)->recv(item->rdata_pkt());
+      Packet *pkt = item->rdata_pkt();
+      if (flag) {
+	pkt = pkt->copy ();
+      } else {
 	flag = 1;
       }
-      else {
-	pkt = item->rdata_pkt()->copy();
-	(*iter)->recv(pkt);
-      }
+      (*iter)->recv(pkt);
 
       iter++;
     }
@@ -673,7 +691,7 @@ NsObject* PgmSender::iface2link (int iface)
 
         sprintf (wrk, "[%s set node_] ifaceGetOutLink %d", name (), iface);
         tcl.evalc (wrk);
-        char* result = tcl.result ();
+        const char* result = tcl.result ();
 #ifdef PGM_DEBUG
 printf ("[iface2link] agent %s\n", result);
 #endif
@@ -685,11 +703,11 @@ NsObject* PgmSender::pkt2agent (Packet *pkt)
 {
         Tcl&            tcl = Tcl::instance();
         char            wrk[64];
-        char            *result;
+        const char            *result;
         int             port;
         NsObject*       agent;
         hdr_ip*         ih = HDR_IP(pkt);
-        nsaddr_t        src = ih->saddr();
+        //nsaddr_t        src = ih->saddr();
 
         port = ih->sport();
 
