@@ -55,9 +55,18 @@
  *   packets.
  */
 
+/* Bugreport about rio.cc:
+ *
+ * David Ros and LuiWei have both reported, in June 2002 and in May 2001,
+ * that rio.cc has a bug in that RIO does not calculate a new average
+ * queue length when an OUT packet arrives.
+ * 
+ * This bug has not been checked or repaired.
+ */
+
 #ifndef lint
 static const char rcsid[] =
-    "@(#) $Header: /nfs/jade/vint/CVSROOT/ns-2/queue/rio.cc,v 1.11 2002/03/21 01:42:36 johnh Exp $ (LBL)";
+    "@(#) $Header: /cvsroot/nsnam/ns-2/queue/rio.cc,v 1.13 2006/12/17 15:21:59 mweigle Exp $ (LBL)";
 #endif
 
 #include "rio.h"
@@ -551,7 +560,8 @@ void RIOQueue::enque(Packet* pkt)
 void
 RIOQueue::trace(TracedVar* v)
 {
-	char wrk[500], *p;
+	char wrk[500];
+	const char *p;
 
 	if (((p = strstr(v->name(), "ave")) == NULL) &&
 	    ((p = strstr(v->name(), "in_ave")) == NULL) &&
