@@ -22,7 +22,7 @@
 #    specific prior written permission.
 # 
 
-# @(#) $Header: /cvsroot/nsnam/ns-2/tcl/lib/ns-default.tcl,v 1.369 2006/10/23 05:33:16 sallyfloyd Exp $
+# @(#) $Header: /cvsroot/nsnam/ns-2/tcl/lib/ns-default.tcl,v 1.372 2007/06/17 21:44:46 tom_henderson Exp $
 
 # THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 # ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -767,7 +767,6 @@ Agent/SCTP set associationMaxRetrans_ 10;# 10 attempts
 Agent/SCTP set pathMaxRetrans_ 5        ;# 5 attempts (per destination)
 Agent/SCTP set changePrimaryThresh_ -1  ;# infinite (ie, never change primary
 Agent/SCTP set maxInitRetransmits_ 8    ;# 8 attempts
-Agent/SCTP set oneHeartbeatTimer_ 1     ;# single heartbeat timer for all dests
 Agent/SCTP set heartbeatInterval_ 30    ;# 30 secs
 Agent/SCTP set mtu_ 1500                ;# MTU of ethernet (most common)
 Agent/SCTP set initialRwnd_ 65536       ;# default inital receiver window
@@ -815,10 +814,15 @@ Agent/SCTP/MfrTimestamp set mfrCount_ 0
 Agent/SCTP/CMT set useCmtReordering_ 1  ;# Turn ON CMT Reordering algo
 Agent/SCTP/CMT set useCmtCwnd_ 1        ;# Turn ON CMT cwnd growth algo
 Agent/SCTP/CMT set useCmtDelAck_ 1      ;# Turn ON CMT delayed ack algo
-Agent/SCTP/CMT set eCmtRtxPolicy_ 1     ;# Default policy = RTX_TO_SAME
+Agent/SCTP/CMT set eCmtRtxPolicy_ 4     ;# Default policy = RTX_CWND
 ## CMT-PF variables
 Agent/SCTP/CMT set useCmtPF_ 0          ;# CMT-PF turned off
-Agent/SCTP/CMT set cmtPFCwnd_ 1	        ;# Cwnd in MTUs after HB-ACK (1 or 2)
+Agent/SCTP/CMT set cmtPFCwnd_ 2	        ;# Cwnd in MTUs after HB-ACK (1 or 2)
+## CMT-PF trace variables
+Agent/SCTP/CMT set countPFToActiveNewData_ 0 ;# count of PF->Active changes
+                                              # for new data transfer
+Agent/SCTP/CMT set countPFToActiveRtxms_ 0;   # count of PF->Active changes
+                                              # for retransmissions
 
 Agent/TCP set seqno_ 0
 Agent/TCP set t_seqno_ 0
@@ -869,7 +873,8 @@ Agent/TCP set slow_start_restart_ true
 Agent/TCP set restart_bugfix_ true
 Agent/TCP set tcpTick_ 0.01 ;		# default changed on 2002/03/07
 					# to reflect a changing reality.
-Agent/TCP set maxrto_ 100000
+Agent/TCP set maxrto_ 60 ; 		# default changed on 2007/03/28
+					#  to reflect RFC2988.
 Agent/TCP set minrto_ 0.2 ;		# Default changed to 200ms on 
 					#  2004/10/14, to match values
 					#  used by many implementations.
@@ -1053,6 +1058,7 @@ Agent/TFRC set voip_max_pkt_rate_ 100 ;  # Max rate in pps, for voip mode.
 Agent/TFRC set fsize_ 1460 ;	# Default size for large TCP packets. 
 				# Used for VoIP mode.
 Agent/TFRC set headersize_ 32 ; # Size for packet headers.
+# End of VoIP mode.
 # Variants in the TFRC algorithms:
 Agent/TFRC set rate_init_option_ 2 ;	# Added on 10/20/2004
 				# Set to 1 for backward compatibility. 
@@ -1062,14 +1068,17 @@ Agent/TFRC set slow_increase_ 1 ;	# Added on 10/20//2004
 				# Set to 1 for gradual rate changes.  
 				# This also gives backward compatibility.
 # Agent/TFRC set ss_changes_ 1 ;	# Deleted on 3/14//2006. 
-Agent/TFRC set maxHeavyRounds_ 1; # Number of rounds for sending rate allowed
+Agent/TFRC set maxHeavyRounds_ 0; # Number of rounds for sending rate allowed
 				  #  to be greater than twice receiving rate.
+				  # Default changed on 3/27/2007, to conform
+				  # to RFC3448 and CCID 3.
 Agent/TFRC set conservative_ 0 ;  # Set to true for a conservative 
 				  # response to heavy congestion.
 Agent/TFRC set scmult_ 1.5 ;	# self clocking parameter for conservative_
 Agent/TFRC set oldCode_ false ; # Set to 1 to use old code for datalimited
 				#   applications.
 				# Parameter added on 12/18/02.
+# End of Variands.
 # Parameters:
 Agent/TFRC set packetSize_ 1000 
 Agent/TFRC set df_ 0.95 ;	# decay factor for accurate RTT estimate
@@ -1098,6 +1107,7 @@ Agent/TFRCSink set NumFeedback_ 1
 Agent/TFRCSink set AdjustHistoryAfterSS_ 1
 Agent/TFRCSink set NumSamples_ -1
 Agent/TFRCSink set discount_ 1;	# History Discounting
+Agent/TFRCSink set minDiscountRatio_ 0.5; # Minimum for history discounting.
 Agent/TFRCSink set printLoss_ 0
 Agent/TFRCSink set smooth_ 1 ;	# smoother Average Loss Interval
 Agent/TFRCSink set ShortIntervals_ 0 ; #  For calculating loss event rates 
