@@ -33,7 +33,7 @@
 # OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 # SUCH DAMAGE.
 #
-# @(#) $Header: /nfs/jade/vint/CVSROOT/ns-2/tcl/lib/ns-packet.tcl,v 1.46 2002/01/14 20:05:43 xuanc Exp $
+# @(#) $Header: /cvsroot/nsnam/ns-2/tcl/lib/ns-packet.tcl,v 1.55 2005/06/17 17:41:16 sfloyd Exp $
 #
 # set up the packet format for the simulation
 # (initial version)
@@ -111,50 +111,65 @@ proc remove-all-packet-headers {} {
 }
 
 foreach prot {
-	AODV
-	ARP
-  	aSRM 
+# Common:
 	Common 
-	CtrMcast 
-	Diffusion
-	Encap
 	Flags
-	HttpInval
-	IMEP
-	IP
-        IPinIP 
-	IVS
-	LDP
-	LL
-	mcastCtrl
-	MFTP
-	MPLS
-	Mac 
-	Message
-        MIP 
-	Ping
-	PGM
-	PGM_SPM
-	PGM_NAK
-	RAP 
-	RTP
-	Resv 
-	rtProtoDV
-	rtProtoLS
-	SR
-	Src_rt
-  	SRM 
-  	SRMEXT
-	Snoop
-	TCP
-	TCPA
-	TFRC
-	TFRC_ACK
-	TORA
-	GAF
-	UMP 
-	Pushback
-	NV
+	IP 	# IP
+# Routing Protocols:
+	NV 	# NixVector classifier for stateless routing 
+	rtProtoDV 	# distance vector routing protocol
+	rtProtoLS 	# link state routing protocol
+	SR 	# source routing, dsr/hdr_sr.cc
+	Src_rt 	# source routing, src_rtg/hdr_src.cc
+# Routers:
+	LDP 	# mpls/ldp.cc
+	MPLS 	# MPLS, MultiProtocol Label Switching
+	Resv 	# Token buckets, for reservations.
+	UMP 	# Admission control, adc/ump.cc
+	Pushback 	# Pushback, router-to-router
+# Multicast:
+  	aSRM 	# mcast/srm.cc
+	CtrMcast 	# Centralized Multicast routing
+	mcastCtrl 	# mcast/mcast_ctrl.cc
+	MFTP 	# Multicast File Transfer Protocol
+	PGM 	# PGM multicast
+	PGM_SPM # PGM multicast
+	PGM_NAK # PGM multicast
+  	SRM 	# SRM, multicast
+  	SRMEXT 	# SRM, multicast
+# Transport Protocols and related protocols:
+	HttpInval 	# HTTP
+	IVS 	# Inria video conferencing system 
+	QS 	# Quick-Start
+	RAP 	# Rate Adaption Protocol, transport protocol.
+	RTP 	# RTP.  Also used for UPD traffic.
+	SCTP 	# SCTP, transport protocol
+	Snoop 	# tcp/snoop.cc
+	TCP 	# TCP, transport protocol
+	TCPA 	# Asymmetric TCP, transport protocol
+	TFRC 	# TFRC, transport protocol
+	TFRC_ACK 	# TFRC, transport protocol
+	XCP 	# XCP, transport protocol
+# Application-Layer Protocols:
+	Message # a protocol to carry text messages
+	Ping 	# Ping
+# Wireless:
+	ARP 	# Address Resolution Protocol, network wireless stack
+	GAF 	# Geographic Adaptive Delity, for ad-hoc networks
+	LL 	# network wireless stack
+        LRWPAN  # zheng, wpan/p802_15_4mac.cc
+	Mac 	# network wireless stack
+# Mobility, Ad-Hoc Networks, Sensor Nets:
+	AODV 	# routing protocol for ad-hoc networks
+	Diffusion 	# diffusion/diffusion.cc
+	IMEP 	# Internet MANET Encapsulation Protocol, for ad-hoc networks
+        MIP 	# Mobile IP, mobile/mip-reg.cc
+	Smac 	# Sensor-MAC
+	TORA 	# routing protocol for ad-hoc networks
+# Other:
+	Encap 	# common/encap.cc
+        IPinIP 	# IP encapsulation 
+	HDLC 	# High Level Data Link Control
 } {
 	add-packet-header $prot
 }
