@@ -3,20 +3,16 @@
 /*
  * Copyright (c) Intel Corporation 2001. All rights reserved.
  *
- * License is granted to copy, to use, and to make and to use derivative
- * works for research and evaluation purposes, provided that Intel is
- * acknowledged in all documentation pertaining to any such copy or 
- * derivative work. Intel grants no other licenses expressed or
- * implied. The Intel trade name should not be used in any advertising
- * without its written permission. 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *   http://www.apache.org/licenses/LICENSE-2.0
  *
- * INTEL CORPORATION MAKES NO REPRESENTATIONS CONCERNING EITHER THE
- * MERCHANTABILITY OF THIS SOFTWARE OR THE SUITABILITY OF THIS SOFTWARE
- * FOR ANY PARTICULAR PURPOSE.  The software is provided "as is" without 
- * express or implied warranty of any kind.
- *
- * These notices must be retained in any copies of any part of this 
- * software. 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
@@ -112,7 +108,7 @@
 
 #ifndef lint
 static const char rcsid[] =
-    "@(#) $Header: /nfs/jade/vint/CVSROOT/ns-2/tcp/tcp-full.cc,v 1.113 2002/07/24 04:34:18 tomh Exp $ (LBL)";
+    "@(#) $Header: /cvsroot/nsnam/ns-2/tcp/tcp-full.cc,v 1.123 2006/06/30 23:18:01 sallyfloyd Exp $ (LBL)";
 #endif
 
 #include "ip.h"
@@ -198,6 +194,8 @@ FullTcpAgent::delay_bind_init_all()
         delay_bind_init_one("open_cwnd_on_pack_");
         delay_bind_init_one("halfclose_");
         delay_bind_init_one("nopredict_");
+        delay_bind_init_one("ecn_syn_");
+        delay_bind_init_one("ecn_syn_wait_");
         delay_bind_init_one("spa_thresh_");
 
 	TcpAgent::delay_bind_init_all();
@@ -226,6 +224,8 @@ FullTcpAgent::delay_bind_dispatch(const char *varName, const char *localName, Tc
         if (delay_bind_bool(varName, localName, "open_cwnd_on_pack_", &open_cwnd_on_pack_, tracer)) return TCL_OK;
         if (delay_bind_bool(varName, localName, "halfclose_", &halfclose_, tracer)) return TCL_OK;
         if (delay_bind_bool(varName, localName, "nopredict_", &nopredict_, tracer)) return TCL_OK;
+        if (delay_bind_bool(varName, localName, "ecn_syn_", &ecn_syn_, tracer)) return TCL_OK;
+        if (delay_bind_bool(varName, localName, "ecn_syn_wait_", &ecn_syn_wait_, tracer)) return TCL_OK;
 
         return TcpAgent::delay_bind_dispatch(varName, localName, tracer);
 }
@@ -558,8 +558,21 @@ FullTcpAgent::flagstr(int hflags)
 		"<ACK>", "<ACK,FIN>", "<ACK,SYN>", "<ACK,SYN,FIN>", // 0x14-0x17
 		"<PSH,ACK>", "<PSH,ACK,FIN>", "<PSH,ACK,SYN>", "<PSH,ACK,SYN,FIN>", // 0x18-0x1b
 	};
-	if (hflags < 0 || (hflags > 28))
-		return ("<invalid>");
+	if (hflags < 0 || (hflags > 28)) {
+		/* Added strings for CWR and ECE  -M. Weigle 6/27/02 */
+		if (hflags == 72) 
+	 		return ("<ECE,PSH>");
+	 	else if (hflags == 80)
+	 		return ("<ECE,ACK>");
+	 	else if (hflags == 88) 
+	 		return ("<ECE,PSH,ACK>");
+	 	else if (hflags == 152) 
+	 		return ("<CWR,PSH,ACK>");
+		else if (hflags == 153)
+			return ("<CWR,PSH,ACK,FIN>");
+		else
+			return ("<invalid>");
+	}
 	return (flagstrs[hflags]);
 }
 
@@ -609,6 +622,9 @@ FullTcpAgent::reset()
 		recent_ = recent_age_ = -1.0;
 
 	fastrecov_ = FALSE;
+
+	closed_ = 0;
+	close_on_empty_ = FALSE;
 }
 
 /*
@@ -764,7 +780,8 @@ int
 FullTcpAgent::pack(Packet *pkt)
 {
 	hdr_tcp *tcph = hdr_tcp::access(pkt);
-	return (tcph->ackno() >= highest_ack_ &&
+	/* Added check for fast recovery.  -M. Weigle 5/2/02 */
+	return (fastrecov_ && tcph->ackno() >= highest_ack_ &&
 		tcph->ackno() < recover_);
 }
 
@@ -806,9 +823,9 @@ FullTcpAgent::ack_action(Packet* p)
  * Also, set the size of the tcp header.
  */
 void
-FullTcpAgent::sendpacket(int seqno, int ackno, int pflags, int datalen, int reason)
+FullTcpAgent::sendpacket(int seqno, int ackno, int pflags, int datalen, int reason, Packet *p)
 {
-        Packet* p = allocpkt();
+        if (!p) p = allocpkt();
         hdr_tcp *tcph = hdr_tcp::access(p);
 	hdr_flags *fh = hdr_flags::access(p);
 
@@ -833,11 +850,21 @@ FullTcpAgent::sendpacket(int seqno, int ackno, int pflags, int datalen, int reas
 	 *	ect_: I am doing ECN (ecn_ should be T and peer does ECN)
 	 */
 
-	// set ect on data packets (not syn or ack packets)
-	if ( datalen > 0 && ecn_ ){
+	if (datalen > 0 && ecn_ ){
+	        // set ect on data packets 
 		fh->ect() = ect_;	// on after mutual agreement on ECT
+        } else if (ecn_ && ecn_syn_ && (pflags & TH_SYN) && (pflags & TH_ACK)) {
+                // set ect on syn/ack packet, if syn packet was negotiating ECT
+                fh->ect() = ect_;
+	} else {
+		/* Set ect() to 0.  -M. Weigle 1/19/05 */
+		fh->ect() = 0;
+	}
+	if (ecn_ && ect_ && recent_ce_ ) { 
+		// This is needed here for the ACK in a SYN, SYN/ACK, ACK
+		// sequence.
+		pflags |= TH_ECE;
 	}
-
         // fill in CWR and ECE bits which don't actually sit in
         // the tcp_flags but in hdr_flags
         if ( pflags & TH_ECE) {
@@ -848,14 +875,10 @@ FullTcpAgent::sendpacket(int seqno, int ackno, int pflags, int datalen, int reas
         if ( pflags & TH_CWR ) {
                 fh->cong_action() = 1;
         }
-
-	//
-	// although CWR bit is ordinarily associated with ECN,
-	// it utility within the simulator for traces.  Thus, set
-	// it even if we aren't doing ECN
-	//
-	if ( datalen > 0 )
-	 	fh->cwr() =  cong_action_;
+	else {
+		/* Set cong_action() to 0  -M. Weigle 1/19/05 */
+		fh->cong_action() = 0;
+	}
 
 	/* actual size is data length plus header length */
 
@@ -971,7 +994,6 @@ FullTcpAgent::foutput(int seqno, int reason)
 		datalen = maxseg_;
 	}
 
-
 	//
 	// this is an option that causes us to slow-start if we've
 	// been idle for a "long" time, where long means a rto or longer
@@ -1068,12 +1090,30 @@ send:
                 pflags |= TH_ECE;
                 pflags &= ~TH_CWR;
         }
-  
-        /* set CWR if necessary */
-        if (ecn_ && ect_ && cong_action_) pflags |= TH_CWR;
+	else if (ecn_ && ect_ && cong_action_ && 
+	             (!is_retransmit || SetCWRonRetransmit_)) 
+		/* 
+		 * Don't set CWR for a retranmitted SYN+ACK (has ecn_ 
+		 * and cong_action_ set) or on any retransmits.
+		 * -M. Weigle 6/19/02
+		 */
+		/* set CWR if necessary */
+		pflags |= TH_CWR;
+
+	/* moved from sendpacket()  -M. Weigle 6/19/02 */
+	//
+	// although CWR bit is ordinarily associated with ECN,
+	// it has utility within the simulator for traces. Thus, set
+	// it even if we aren't doing ECN
+	//
+	if (datalen > 0 && cong_action_ && !is_retransmit) {
+		pflags |= TH_CWR;
+	}
   
         /* set ECE if necessary */
-        if (ecn_ && ect_ && recent_ce_ ) pflags |= TH_ECE;
+        if (ecn_ && ect_ && recent_ce_ ) {
+		pflags |= TH_ECE;
+	}
 
         /* 
          * Tack on the FIN flag to the data segment if close_on_empty_
@@ -1107,7 +1147,11 @@ send:
 	 */
 
 	int reliable = datalen + syn + fin; // seq #'s reliably sent
-	if (cong_action_ && reliable > 0)
+	/* 
+	 * Don't reset cong_action_ until we send new data.
+	 * -M. Weigle 6/19/02
+	 */
+	if (cong_action_ && reliable > 0 && !is_retransmit)
 		cong_action_ = FALSE;
 
 	// highest: greatest sequence number sent + 1
@@ -1266,6 +1310,14 @@ FullTcpAgent::newack(Packet* pkt)
 			recent_age_ = now();
 			recent_ = tcph->ts();
 			rtt_update(now() - tcph->ts_echo());
+			if (ts_resetRTO_ && (!ect_ || !ecn_backoff_ ||
+		           !hdr_flags::access(pkt)->ecnecho())) { 
+				// From Andrei Gurtov
+				//
+                         	// Don't end backoff if still in ECN-Echo with
+                         	// a congestion window of 1 packet.
+				t_backoff_ = 1;
+			}
 		} else if (rtt_active_ && ackno > rtt_seq_) {
 			// got an RTT sample, record it
 			// "t_backoff_ = 1;" deleted by T. Kelly.
@@ -1453,6 +1505,17 @@ FullTcpAgent::recv(Packet *pkt, Handler*)
 //prpkt(pkt);
 //}
 
+	/* 
+	 * Acknowledge FIN from passive closer even in TCPS_CLOSED state
+	 * (since we lack TIME_WAIT state and RST packets,
+	 * the loss of the FIN packet from the passive closer will make that
+	 * endpoint retransmit the FIN forever)
+	 * -F. Hernandez-Campos 8/6/00
+	 */
+	if ( (state_ == TCPS_CLOSED) && (tiflags & TH_FIN) ) {
+		goto dropafterack;
+	}
+
 	/*
 	 * Don't expect to see anything while closed
 	 */
@@ -1468,7 +1531,6 @@ FullTcpAgent::recv(Packet *pkt, Handler*)
          * Process options if not in LISTEN state,
          * else do it below
          */
-
 	if (state_ != TCPS_LISTEN)
 		dooptions(pkt);
 
@@ -1479,30 +1541,16 @@ FullTcpAgent::recv(Packet *pkt, Handler*)
 	 * at time t0 = (0.0 + k * interval_) for some k such
 	 * that t0 > now
 	 */
-
 	if (delack_interval_ > 0.0 &&
 	    (delack_timer_.status() != TIMER_PENDING)) {
 		int last = int(now() / delack_interval_);
 		delack_timer_.resched(delack_interval_ * (last + 1.0) - now());
 	}
 
-	/*
-	 * sanity check for ECN: shouldn't be seeing a CE bit if
-	 * ECT wasn't set on the packet first.  If we see this, we
-	 * probably have a misbehaving router...
-	 */
-
-	if (fh->ce() && !fh->ect()) {
-	    fprintf(stderr,
-	    "%f: FullTcpAgent::recv(%s): warning: CE bit on, but ECT false!\n",
-		now(), name());
-	}
-
 	/*
 	 * Try header prediction: in seq data or in seq pure ACK
 	 *	with no funny business
 	 */
-
 	if (!nopredict_ && predict_ok(pkt)) {
                 /*
                  * If last ACK falls within this segment's sequence numbers,
@@ -1524,17 +1572,18 @@ FullTcpAgent::recv(Packet *pkt, Handler*)
 		// generate a stream of ecnecho bits until we see a true
 		// cong_action bit
 		//
-		if (ecn_) {
-			if (fh->ce() && fh->ect()) {
-				// no CWR from peer yet... arrange to
-				// keep sending ECNECHO
-				recent_ce_ = TRUE;
-			} else if (fh->cwr()) {
-				// got CWR response from peer.. stop
-				// sending ECNECHO bits
-				recent_ce_ = FALSE;
-			}
-		}
+
+	    	if (ecn_) {
+	    		if (fh->ce() && fh->ect()) {
+	    			// no CWR from peer yet... arrange to
+	    			// keep sending ECNECHO
+	    			recent_ce_ = TRUE;
+	    		} else if (fh->cwr()) {
+	    			// got CWR response from peer.. stop
+	    			// sending ECNECHO bits
+	    			recent_ce_ = FALSE;
+	    		}
+	    	}
 
 		// Header predication basically looks to see
 		// if the incoming packet is an expected pure ACK
@@ -1635,7 +1684,6 @@ FullTcpAgent::recv(Packet *pkt, Handler*)
 
 		// check for a ECN-SYN with ECE|CWR
 		if (ecn_ && fh->ecnecho() && fh->cong_action()) {
-
 			ect_ = TRUE;
 		}
 
@@ -1693,10 +1741,11 @@ cancel_rtx_timer();	// cancel timer on our 1st SYN [does this belong!?]
 
 		if (tiflags & TH_ACK) {
 			// SYN+ACK (our SYN was acked)
-			// CHECKME
-                        // Check ECN-SYN+ACK packet
-                        if (ecn_ && fh->ecnecho() && !fh->cong_action())
+                        if (ecn_ && fh->ecnecho() && !fh->cong_action()) {
                                 ect_ = TRUE;
+	    		        if ( fh->ce() ) 
+	    				recent_ce_ = TRUE;
+	    		}
 			highest_ack_ = ackno;
 			cwnd_ = initial_window();
 
@@ -1776,6 +1825,15 @@ trimthenstep6:
 		goto step6;
 
 	case TCPS_LAST_ACK:
+		/* 
+		 * The only way we're in LAST_ACK is if we've already
+		 * received a FIN, so ignore all retranmitted FINS.
+		 * -M. Weigle 7/23/02
+		 */
+		if (tiflags & TH_FIN) {
+			goto drop;
+		}
+		break;
 	case TCPS_CLOSING:
 		break;
 	} /* end switch(state_) */
@@ -1913,9 +1971,19 @@ trimthenstep6:
 	}
 
 	if ((tiflags & TH_ACK) == 0) {
-		fprintf(stderr, "%f: FullTcpAgent::recv(%s) got packet lacking ACK (state:%d): ",
-			now(), name(), state_);
-		prpkt(pkt);
+		/*
+		 * Added check for state != SYN_RECEIVED.  We will receive a 
+		 * duplicate SYN in SYN_RECEIVED when our SYN/ACK was dropped.
+		 * We should just ignore the duplicate SYN (our timeout for 
+		 * resending the SYN/ACK is about the same as the client's 
+		 * timeout for resending the SYN), but give no error message. 
+		 * -M. Weigle 07/24/01
+		 */
+		if (state_ != TCPS_SYN_RECEIVED) {
+			fprintf(stderr, "%f: FullTcpAgent::recv(%s) got packet lacking ACK (state:%d): ",
+				now(), name(), state_);
+			prpkt(pkt);
+		}
 		goto drop;
 	}
 
@@ -1944,7 +2012,17 @@ trimthenstep6:
                 } else {
                         newstate(TCPS_ESTABLISHED);
                 }
-		cwnd_ = initial_window();
+		if (ecn_ && ect_ && ecn_syn_ && fh->ecnecho() )
+			// The SYN/ACK packet was ECN-marked.
+			if (ecn_syn_wait_) {
+				// A timer will be called in ecn().
+				cwnd_ = 1;
+				use_rtt_ = 1;
+			} else
+			        // Congestion window will be halved in ecn().
+				cwnd_ = 2;
+		else 
+			cwnd_ = initial_window();
 		/* fall into ... */
 
         /*
@@ -2036,7 +2114,6 @@ trimthenstep6:
 				} else if (++dupacks_ == tcprexmtthresh_) {
 					// ACK at highest_ack_ AND meets threshold
 					//trace_event("FAST_RECOVERY");
-
 					dupack_action(); // maybe fast rexmt
 					goto drop;
 
@@ -2111,7 +2188,6 @@ process_ACK:
 		 * if this is an ACK with an ECN indication, handle this
 		 * but not if it is a syn packet
 		 */
-
 		if (fh->ecnecho() && !(tiflags&TH_SYN) )
 		if (fh->ecnecho()) {
 			// Note from Sally: In one-way TCP,
@@ -2372,6 +2448,7 @@ drop:
    	Packet::free(pkt);
 	return;
 }
+
 /*  
  * Dupack-action: what to do on a DUP ACK.  After the initial check
  * of 'recover' below, this function implements the following truth
@@ -2398,10 +2475,9 @@ FullTcpAgent::dupack_action()
 	fastrecov_ = TRUE;
 	rtxbytes_ = 0;
 
-        if (recovered || (!bug_fix_ && !ecn_) ||
-	    // Q: is last_cwnd_action == dupack enough to trigger?
-	    // probably not.  may need to fix 1-way reno model as well
-	    last_cwnd_action_ == CWND_ACTION_DUPACK) {
+        if (recovered || (!bug_fix_ && !ecn_) 
+            || (last_cwnd_action_ == CWND_ACTION_DUPACK)
+            || ( highest_ack_ == 0)) {
                 goto full_reno_action;
         }       
     
@@ -2478,7 +2554,11 @@ void
 FullTcpAgent::timeout(int tno)
 {
 
-	if (state_ == TCPS_CLOSED || state_ == TCPS_LISTEN) {
+	/*
+	 * Due to F. Hernandez-Campos' fix in recv(), we may send an ACK
+	 * while in the CLOSED state.  -M. Weigle 7/24/01
+	 */
+	if (state_ == TCPS_LISTEN) {
 	 	// shouldn't be getting timeouts here
 		fprintf(stderr, "%f: FullTcpAgent(%s): unexpected timeout %d in state %s\n",
 			now(), name(), tno, statestr(state_));
@@ -2584,9 +2664,7 @@ TahoeFullTcpAgent::dupack_action()
 	fastrecov_ = TRUE;
 	rtxbytes_ = 0;
 
-        if (recovered || (!bug_fix_ && !ecn_) ||
-	    // Q: is last_cwnd_action dupack enough?
-            last_cwnd_action_ == CWND_ACTION_DUPACK) {
+        if (recovered || (!bug_fix_ && !ecn_) || highest_ack_ == 0) {
                 goto full_tahoe_action;
         }
    
@@ -2675,7 +2753,8 @@ void
 SackFullTcpAgent::reset()
 {
 	sq_.clear();			// no SACK blocks
-	sack_min_ = h_seqno_ -1;	// no left edge of SACK blocks
+	/* Fixed typo.  -M. Weigle 6/17/02 */
+	sack_min_ = h_seqno_ = -1;	// no left edge of SACK blocks
 	FullTcpAgent::reset();
 }
 
@@ -2710,12 +2789,20 @@ SackFullTcpAgent::dupack_action()
         }           
 
         if (ecn_ && last_cwnd_action_ == CWND_ACTION_ECN) {
+		/* 
+		 * Received ECN notification and 3 DUPACKs in same 
+		 * window. Don't cut cwnd again, but retransmit lost
+		 * packet.   -M. Weigle  6/19/02
+		 */
 		last_cwnd_action_ = CWND_ACTION_DUPACK;
-                cancel_rtx_timer();
-                rtt_active_ = FALSE;
-		send_much(1, REASON_DUPACK, maxburst_);
-                return; 
-        }               
+		cancel_rtx_timer();
+		rtt_active_ = FALSE;
+		int amt = fast_retransmit(highest_ack_);
+		pipectrl_ = TRUE;
+		h_seqno_ = highest_ack_ + amt;
+		send_much(0, REASON_DUPACK, maxburst_);
+		return; 
+	}
    
         if (bug_fix_) {
                 /*                              
@@ -2773,6 +2860,13 @@ SackFullTcpAgent::ack_action(Packet* p)
                 sq_.cleartonxt();
         }
 	dupacks_ = 0;
+
+	/* 
+	 * Update h_seqno_ on new ACK (same as for partial ACKS)
+	 * -M. Weigle 6/3/05
+	 */
+	if (h_seqno_ < highest_ack_)
+		h_seqno_ = highest_ack_;
 }
 
 //
@@ -2834,7 +2928,9 @@ SackFullTcpAgent::process_sack(hdr_tcp* tcph)
 
 	int slen = tcph->sa_length(), i;
 	for (i = 0; i < slen; ++i) {
-		if (tcph->sa_left(i) >= tcph->sa_right(i)) {
+		/* Added check for FIN   -M. Weigle 5/21/02 */
+		if ((tcph->flags() & TH_FIN == 0) && 
+		    tcph->sa_left(i) >= tcph->sa_right(i)) {
 			fprintf(stderr,
 			    "%f: FullTcpAgent(%s) warning: received illegal SACK block [%d,%d]\n",
 				now(), name(), tcph->sa_left(i), tcph->sa_right(i));
@@ -2861,6 +2957,13 @@ SackFullTcpAgent::send_allowed(int seq)
 		return FALSE;
 	}
 
+	/*
+	 * If not in ESTABLISHED, don't send anything we don't have
+	 *   -M. Weigle 7/18/02
+	 */
+	if (state_ != TCPS_ESTABLISHED && seq > curseq_)
+		return FALSE;
+
 	// don't overshoot cwnd_
 	int cwin = int(cwnd_) * maxseg_;
 	return (pipe_ < cwin);
