@@ -48,7 +48,7 @@ public:
 } class_tcpsink;
 
 Acker::Acker() : next_(0), maxseen_(0), wndmask_(MWM), ecn_unacked_(0), 
-	ts_to_echo_(0)
+	ts_to_echo_(0), last_ack_sent_(0)
 {
 	seen_ = new int[MWS];
 	memset(seen_, 0, (sizeof(int) * (MWS)));
@@ -84,10 +84,15 @@ void Acker::resize_buffers(int sz) {
 	return; 
 }
 
-void Acker::update_ts(int seqno, double ts)
+void Acker::update_ts(int seqno, double ts, int rfc1323)
 {
-	if (ts >= ts_to_echo_ && seqno <= next_)
-		ts_to_echo_ = ts;
+	// update timestamp if segment advances with ACK.
+        // Code changed by Andrei Gurtov.
+        if (rfc1323 && seqno == last_ack_sent_ + 1)
+               ts_to_echo_ = ts;
+        else if (ts >= ts_to_echo_ && seqno <= last_ack_sent_ + 1)
+               //rfc1323-bis, update timestamps from duplicate segments
+               ts_to_echo_ = ts;
 }
 
 // returns number of bytes that can be "delivered" to application
@@ -175,6 +180,7 @@ TcpSink::TcpSink(Acker* acker) : Agent(PT_ACK), acker_(acker), save_(NULL),
 	lastreset_(0.0)
 {
 	bytes_ = 0; 
+	bind("bytes_", &bytes_);
 
 	/*
 	 * maxSackBlocks_ does wierd tracing things.
@@ -191,10 +197,12 @@ TcpSink::delay_bind_init_all()
 {
         delay_bind_init_one("packetSize_");
         delay_bind_init_one("ts_echo_bugfix_");
+	delay_bind_init_one("ts_echo_rfc1323_");
 	delay_bind_init_one("bytes_"); // For throughput measurements in JOBS
         delay_bind_init_one("generateDSacks_"); // used only by sack
 	delay_bind_init_one("qs_enabled_");
 	delay_bind_init_one("RFC2581_immediate_ack_");
+	delay_bind_init_one("ecn_syn_");
 #if defined(TCP_DELAY_BIND_ALL) && 0
         delay_bind_init_one("maxSackBlocks_");
 #endif /* TCP_DELAY_BIND_ALL */
@@ -207,9 +215,11 @@ TcpSink::delay_bind_dispatch(const char *varName, const char *localName, TclObje
 {
         if (delay_bind(varName, localName, "packetSize_", &size_, tracer)) return TCL_OK;
         if (delay_bind_bool(varName, localName, "ts_echo_bugfix_", &ts_echo_bugfix_, tracer)) return TCL_OK;
+	if (delay_bind_bool(varName, localName, "ts_echo_rfc1323_", &ts_echo_rfc1323_, tracer)) return TCL_OK;
         if (delay_bind_bool(varName, localName, "generateDSacks_", &generate_dsacks_, tracer)) return TCL_OK;
         if (delay_bind_bool(varName, localName, "qs_enabled_", &qs_enabled_, tracer)) return TCL_OK;
         if (delay_bind_bool(varName, localName, "RFC2581_immediate_ack_", &RFC2581_immediate_ack_, tracer)) return TCL_OK;
+	if (delay_bind_bool(varName, localName, "ecn_syn_", &ecn_syn_ ,tracer)) return TCL_OK;
 #if defined(TCP_DELAY_BIND_ALL) && 0
         if (delay_bind(varName, localName, "maxSackBlocks_", &max_sack_blocks_, tracer)) return TCL_OK;
 #endif /* TCP_DELAY_BIND_ALL */
@@ -257,7 +267,6 @@ void TcpSink::ack(Packet* opkt)
 	// opkt is the "old" packet that was received
 	// npkt is the "new" packet being constructed (for the ACK)
 	double now = Scheduler::instance().clock();
-	hdr_flags *sf;
 
 	hdr_tcp *otcp = hdr_tcp::access(opkt);
 	hdr_ip *oiph = hdr_ip::access(opkt);
@@ -270,7 +279,7 @@ void TcpSink::ack(Packet* opkt)
 	        if (otcp->seqno() == 0 && oqsh->flag() == QS_REQUEST) {
 	                nqsh->flag() = QS_RESPONSE;
 	                nqsh->ttl() = (oiph->ttl() - oqsh->ttl()) % 256;
-	                nqsh->rate() = (oqsh->rate() < MWS) ? oqsh->rate() : MWS;
+	                nqsh->rate() = oqsh->rate(); 
 	        }
 	        else {
 	                nqsh->flag() = QS_DISABLE;
@@ -299,30 +308,40 @@ void TcpSink::ack(Packet* opkt)
 	
 	hdr_flags* of = hdr_flags::access(opkt);
 	hdr_flags* nf = hdr_flags::access(npkt);
-	if (save_ != NULL) 
+	hdr_flags* sf;
+	if (save_ != NULL)
 		sf = hdr_flags::access(save_);
+	else 
+		sf = 0;
 		// Look at delayed packet being acked. 
-	if ( (save_ != NULL && sf->cong_action()) || of->cong_action() ) 
+	if ( (sf != 0 && sf->cong_action()) || of->cong_action() ) 
 		// Sender has responsed to congestion. 
 		acker_->update_ecn_unacked(0);
-	if ( (save_ != NULL && sf->ect() && sf->ce())  || 
+	if ( (sf != 0 && sf->ect() && sf->ce())  || 
 			(of->ect() && of->ce()) )
 		// New report of congestion.  
 		acker_->update_ecn_unacked(1);
-	if ( (save_ != NULL && sf->ect()) || of->ect() )
+	if ( (sf != 0 && sf->ect()) || of->ect() )
 		// Set EcnEcho bit.  
 		nf->ecnecho() = acker_->ecn_unacked();
 	if (!of->ect() && of->ecnecho() ||
-		(save_ != NULL && !sf->ect() && sf->ecnecho()) ) 
+		(sf != 0 && !sf->ect() && sf->ecnecho()) ) {
 		 // This is the negotiation for ECN-capability.
 		 // We are not checking for of->cong_action() also. 
 		 // In this respect, this does not conform to the 
 		 // specifications in the internet draft 
 		nf->ecnecho() = 1;
+		if (ecn_syn_) 
+			nf->ect() = 1;
+	}
 	acker_->append_ack(hdr_cmn::access(npkt),
 			   ntcp, otcp->seqno());
 	add_to_ack(npkt);
 	// the above function is used in TcpAsymSink
+
+        // Andrei Gurtov
+        acker_->last_ack_sent_ = ntcp->seqno();
+        // printf("ACK %d ts %f\n", ntcp->seqno(), ntcp->ts_echo());
 	
 	send(npkt, 0);
 	// send it
@@ -346,15 +365,17 @@ void TcpSink::recv(Packet* pkt, Handler*)
 		Packet::free(pkt);
 		return;
 	}
-	acker_->update_ts(th->seqno(),th->ts());
+	acker_->update_ts(th->seqno(),th->ts(),ts_echo_rfc1323_);
 	// update the timestamp to echo
 	
       	numToDeliver = acker_->update(th->seqno(), numBytes);
 	// update the recv window; figure out how many in-order-bytes
 	// (if any) can be removed from the window and handed to the
 	// application
-	if (numToDeliver)
+	if (numToDeliver) {
+		bytes_ += numToDeliver;
 		recvBytes(numToDeliver);
+	}
 	// send any packets to the application
       	ack(pkt);
 	// ACK the packet
@@ -373,7 +394,8 @@ public:
 DelAckSink::DelAckSink(Acker* acker) : TcpSink(acker), delay_timer_(this)
 {
 	bind_time("interval_", &interval_);
-	bind("bytes_", &bytes_); // useby JOBS
+	// Deleted the line below, since this is bound in TcpSink.
+	// bind("bytes_", &bytes_); // useby JOBS
 }
 
 void DelAckSink::reset() {
@@ -393,7 +415,7 @@ void DelAckSink::recv(Packet* pkt, Handler*)
 		Packet::free(pkt);
 		return;
 	}
-	acker_->update_ts(th->seqno(),th->ts());
+	acker_->update_ts(th->seqno(),th->ts(),ts_echo_rfc1323_);
 	numToDeliver = acker_->update(th->seqno(), numBytes);
 	if (numToDeliver) {
                 bytes_ += numToDeliver; // for JOBS
