@@ -34,7 +34,7 @@
 
 #ifndef lint
 static const char rcsid[] =
-    "@(#) $Header: /nfs/jade/vint/CVSROOT/ns-2/tcp/tcp.cc,v 1.144 2003/02/12 04:16:09 sfloyd Exp $ (LBL)";
+    "@(#) $Header: /cvsroot/nsnam/ns-2/tcp/tcp.cc,v 1.174 2007/03/29 04:56:46 sallyfloyd Exp $ (LBL)";
 #endif
 
 #include <stdlib.h>
@@ -65,21 +65,22 @@ public:
 	}
 } class_tcp;
 
-TcpAgent::TcpAgent() : Agent(PT_TCP), 
-	t_seqno_(0), t_rtt_(0), t_srtt_(0), t_rttvar_(0), 
-	t_backoff_(0), ts_peer_(0), 
-	rtx_timer_(this), delsnd_timer_(this), 
-	burstsnd_timer_(this), 
-	dupacks_(0), curseq_(0), highest_ack_(0), cwnd_(0), ssthresh_(0), 
-	count_(0), fcnt_(0), rtt_active_(0), rtt_seq_(-1), rtt_ts_(0.0), 
-	maxseq_(0), cong_action_(0), ecn_burst_(0), ecn_backoff_(0),
-        ect_(0), lastreset_(0.0),
-        restart_bugfix_(1), closed_(0), nrexmit_(0),
-	first_decrease_(1), qs_requested_(0), qs_approved_(0)
+TcpAgent::TcpAgent() 
+	: Agent(PT_TCP), 
+	  t_seqno_(0), dupacks_(0), curseq_(0), highest_ack_(0), 
+          cwnd_(0), ssthresh_(0), maxseq_(0), count_(0), 
+          rtt_active_(0), rtt_seq_(-1), rtt_ts_(0.0), 
+          lastreset_(0.0), closed_(0), t_rtt_(0), t_srtt_(0), t_rttvar_(0), 
+	  t_backoff_(0), ts_peer_(0), ts_echo_(0), tss(NULL), tss_size_(100), 
+	  rtx_timer_(this), delsnd_timer_(this), burstsnd_timer_(this), 
+	  first_decrease_(1), fcnt_(0), nrexmit_(0), restart_bugfix_(1), 
+          cong_action_(0), ecn_burst_(0), ecn_backoff_(0), ect_(0), 
+          use_rtt_(0), qs_requested_(0), qs_approved_(0),
+	  qs_window_(0), qs_cwnd_(0), frto_(0)
 {
 #ifdef TCP_DELAY_BIND_ALL
+        // defined since Dec 1999.
 #else /* ! TCP_DELAY_BIND_ALL */
-	// not delay-bound because delay-bound tracevars aren't yet supported
 	bind("t_seqno_", &t_seqno_);
 	bind("rtt_", &t_rtt_);
 	bind("srtt_", &t_srtt_);
@@ -99,6 +100,7 @@ TcpAgent::TcpAgent() : Agent(PT_TCP),
         bind("nrexmitbytes_", &nrexmitbytes_);
         bind("necnresponses_", &necnresponses_);
         bind("ncwndcuts_", &ncwndcuts_);
+	bind("ncwndcuts1_", &ncwndcuts1_);
 #endif /* TCP_DELAY_BIND_ALL */
 
 }
@@ -120,21 +122,28 @@ TcpAgent::delay_bind_init_all()
         delay_bind_init_one("overhead_");
         delay_bind_init_one("tcpTick_");
         delay_bind_init_one("ecn_");
+        delay_bind_init_one("SetCWRonRetransmit_");
         delay_bind_init_one("old_ecn_");
+        delay_bind_init_one("bugfix_ss_");
         delay_bind_init_one("eln_");
         delay_bind_init_one("eln_rxmit_thresh_");
         delay_bind_init_one("packetSize_");
         delay_bind_init_one("tcpip_base_hdr_size_");
 	delay_bind_init_one("ts_option_size_");
         delay_bind_init_one("bugFix_");
+	delay_bind_init_one("bugFix_ack_");
+	delay_bind_init_one("bugFix_ts_");
 	delay_bind_init_one("lessCareful_");
         delay_bind_init_one("slow_start_restart_");
         delay_bind_init_one("restart_bugfix_");
         delay_bind_init_one("timestamps_");
+	delay_bind_init_one("ts_resetRTO_");
         delay_bind_init_one("maxburst_");
+	delay_bind_init_one("aggressive_maxburst_");
         delay_bind_init_one("maxcwnd_");
 	delay_bind_init_one("numdupacks_");
 	delay_bind_init_one("numdupacksFrac_");
+	delay_bind_init_one("exitFastRetrans_");
         delay_bind_init_one("maxrto_");
 	delay_bind_init_one("minrto_");
         delay_bind_init_one("srtt_init_");
@@ -163,12 +172,22 @@ TcpAgent::delay_bind_init_all()
 	delay_bind_init_one("high_p_");
 	delay_bind_init_one("high_decrease_");
 	delay_bind_init_one("max_ssthresh_");
-	delay_bind_init_one("cwnd_frac_");
+	delay_bind_init_one("cwnd_range_");
 	delay_bind_init_one("timerfix_");
 	delay_bind_init_one("rfc2988_");
 	delay_bind_init_one("singledup_");
+	delay_bind_init_one("LimTransmitFix_");
 	delay_bind_init_one("rate_request_");
 	delay_bind_init_one("qs_enabled_");
+	delay_bind_init_one("tcp_qs_recovery_");
+	delay_bind_init_one("qs_request_mode_");
+	delay_bind_init_one("qs_thresh_");
+	delay_bind_init_one("qs_rtt_");
+	delay_bind_init_one("print_request_");
+
+	delay_bind_init_one("frto_enabled_");
+	delay_bind_init_one("sfrto_enabled_");
+	delay_bind_init_one("spurious_response_");
 
 #ifdef TCP_DELAY_BIND_ALL
 	// out because delay-bound tracevars aren't yet supported
@@ -191,6 +210,7 @@ TcpAgent::delay_bind_init_all()
         delay_bind_init_one("nrexmitbytes_");
         delay_bind_init_one("necnresponses_");
         delay_bind_init_one("ncwndcuts_");
+	delay_bind_init_one("ncwndcuts1_");
 #endif /* TCP_DELAY_BIND_ALL */
 
 	Agent::delay_bind_init_all();
@@ -212,21 +232,28 @@ TcpAgent::delay_bind_dispatch(const char *varName, const char *localName, TclObj
         if (delay_bind(varName, localName, "overhead_", &overhead_, tracer)) return TCL_OK;
         if (delay_bind(varName, localName, "tcpTick_", &tcp_tick_, tracer)) return TCL_OK;
         if (delay_bind_bool(varName, localName, "ecn_", &ecn_, tracer)) return TCL_OK;
+        if (delay_bind_bool(varName, localName, "SetCWRonRetransmit_", &SetCWRonRetransmit_, tracer)) return TCL_OK;
         if (delay_bind_bool(varName, localName, "old_ecn_", &old_ecn_ , tracer)) return TCL_OK;
+        if (delay_bind_bool(varName, localName, "bugfix_ss_", &bugfix_ss_ , tracer)) return TCL_OK;
         if (delay_bind(varName, localName, "eln_", &eln_ , tracer)) return TCL_OK;
         if (delay_bind(varName, localName, "eln_rxmit_thresh_", &eln_rxmit_thresh_ , tracer)) return TCL_OK;
         if (delay_bind(varName, localName, "packetSize_", &size_ , tracer)) return TCL_OK;
         if (delay_bind(varName, localName, "tcpip_base_hdr_size_", &tcpip_base_hdr_size_, tracer)) return TCL_OK;
 	if (delay_bind(varName, localName, "ts_option_size_", &ts_option_size_, tracer)) return TCL_OK;
         if (delay_bind_bool(varName, localName, "bugFix_", &bug_fix_ , tracer)) return TCL_OK;
+	if (delay_bind_bool(varName, localName, "bugFix_ack_", &bugfix_ack_, tracer)) return TCL_OK;
+	if (delay_bind_bool(varName, localName, "bugFix_ts_", &bugfix_ts_ , tracer)) return TCL_OK;
         if (delay_bind_bool(varName, localName, "lessCareful_", &less_careful_ , tracer)) return TCL_OK;
         if (delay_bind_bool(varName, localName, "timestamps_", &ts_option_ , tracer)) return TCL_OK;
+        if (delay_bind_bool(varName, localName, "ts_resetRTO_", &ts_resetRTO_, tracer)) return TCL_OK;
         if (delay_bind_bool(varName, localName, "slow_start_restart_", &slow_start_restart_ , tracer)) return TCL_OK;
         if (delay_bind_bool(varName, localName, "restart_bugfix_", &restart_bugfix_ , tracer)) return TCL_OK;
         if (delay_bind(varName, localName, "maxburst_", &maxburst_ , tracer)) return TCL_OK;
+        if (delay_bind_bool(varName, localName, "aggressive_maxburst_", &aggressive_maxburst_ , tracer)) return TCL_OK;
         if (delay_bind(varName, localName, "maxcwnd_", &maxcwnd_ , tracer)) return TCL_OK;
 	if (delay_bind(varName, localName, "numdupacks_", &numdupacks_, tracer)) return TCL_OK;
 	if (delay_bind(varName, localName, "numdupacksFrac_", &numdupacksFrac_, tracer)) return TCL_OK;
+	if (delay_bind_bool(varName, localName, "exitFastRetrans_", &exitFastRetrans_, tracer)) return TCL_OK;
         if (delay_bind(varName, localName, "maxrto_", &maxrto_ , tracer)) return TCL_OK;
 	if (delay_bind(varName, localName, "minrto_", &minrto_ , tracer)) return TCL_OK;
         if (delay_bind(varName, localName, "srtt_init_", &srtt_init_ , tracer)) return TCL_OK;
@@ -256,13 +283,21 @@ TcpAgent::delay_bind_dispatch(const char *varName, const char *localName, TclObj
 	if (delay_bind(varName, localName, "high_p_", &high_p_, tracer)) return TCL_OK;
 	if (delay_bind(varName, localName, "high_decrease_", &high_decrease_, tracer)) return TCL_OK;
 	if (delay_bind(varName, localName, "max_ssthresh_", &max_ssthresh_, tracer)) return TCL_OK;
-	if (delay_bind(varName, localName, "cwnd_frac_", &cwnd_frac_, tracer)) return TCL_OK;
+	if (delay_bind(varName, localName, "cwnd_range_", &cwnd_range_, tracer)) return TCL_OK;
 	if (delay_bind_bool(varName, localName, "timerfix_", &timerfix_, tracer)) return TCL_OK;
 	if (delay_bind_bool(varName, localName, "rfc2988_", &rfc2988_, tracer)) return TCL_OK;
         if (delay_bind(varName, localName, "singledup_", &singledup_ , tracer)) return TCL_OK;
+        if (delay_bind_bool(varName, localName, "LimTransmitFix_", &LimTransmitFix_ , tracer)) return TCL_OK;
         if (delay_bind(varName, localName, "rate_request_", &rate_request_ , tracer)) return TCL_OK;
-        if (delay_bind_bool(varName, localName, "qs_enabled_", &qs_enabled_, tracer)) return TCL_OK;
-
+        if (delay_bind_bool(varName, localName, "qs_enabled_", &qs_enabled_ , tracer)) return TCL_OK;
+	if (delay_bind_bool(varName, localName, "tcp_qs_recovery_", &tcp_qs_recovery_, tracer)) return TCL_OK;
+	if (delay_bind(varName, localName, "qs_request_mode_", &qs_request_mode_, tracer)) return TCL_OK;
+	if (delay_bind(varName, localName, "qs_thresh_", &qs_thresh_, tracer)) return TCL_OK;
+	if (delay_bind(varName, localName, "qs_rtt_", &qs_rtt_, tracer)) return TCL_OK;
+	if (delay_bind_bool(varName, localName, "print_request_", &print_request_, tracer)) return TCL_OK;
+	if (delay_bind_bool(varName, localName, "frto_enabled_", &frto_enabled_, tracer)) return TCL_OK;
+	if (delay_bind_bool(varName, localName, "sfrto_enabled_", &sfrto_enabled_, tracer)) return TCL_OK;
+	if (delay_bind_bool(varName, localName, "spurious_response_", &spurious_response_, tracer)) return TCL_OK;
 
 #ifdef TCP_DELAY_BIND_ALL
 	// not if (delay-bound delay-bound tracevars aren't yet supported
@@ -286,81 +321,81 @@ TcpAgent::delay_bind_dispatch(const char *varName, const char *localName, TclObj
         if (delay_bind(varName, localName, "nrexmitbytes_", &nrexmitbytes_ , tracer)) return TCL_OK;
         if (delay_bind(varName, localName, "necnresponses_", &necnresponses_ , tracer)) return TCL_OK;
         if (delay_bind(varName, localName, "ncwndcuts_", &ncwndcuts_ , tracer)) return TCL_OK;
+ 	if (delay_bind(varName, localName, "ncwndcuts1_", &ncwndcuts1_ , tracer)) return TCL_OK;
+
 #endif
 
         return Agent::delay_bind_dispatch(varName, localName, tracer);
 }
 
+#define TCP_WRK_SIZE		512
 /* Print out all the traced variables whenever any one is changed */
 void
 TcpAgent::traceAll() {
+	if (!channel_)
+		return;
+
 	double curtime;
 	Scheduler& s = Scheduler::instance();
-	char wrk[500];
-	int n;
+	char wrk[TCP_WRK_SIZE];
 
 	curtime = &s ? s.clock() : 0;
-	sprintf(wrk,"time: %-8.5f saddr: %-2d sport: %-2d daddr: %-2d dport:"
-		" %-2d maxseq: %-4d hiack: %-4d seqno: %-4d cwnd: %-6.3f"
-		" ssthresh: %-3d dupacks: %-2d rtt: %-6.3f srtt: %-6.3f"
-		" rttvar: %-6.3f bkoff: %-d", curtime, addr(), port(),
-		daddr(), dport(), int(maxseq_), int(highest_ack_),
-		int(t_seqno_), double(cwnd_), int(ssthresh_),
-		int(dupacks_), int(t_rtt_)*tcp_tick_, 
-		(int(t_srtt_) >> T_SRTT_BITS)*tcp_tick_, 
-		int(t_rttvar_)*tcp_tick_/4.0, int(t_backoff_)); 
-	n = strlen(wrk);
-	wrk[n] = '\n';
-	wrk[n+1] = 0;
-	if (channel_)
-		(void)Tcl_Write(channel_, wrk, n+1);
-	wrk[n] = 0;
-	return;
+	snprintf(wrk, TCP_WRK_SIZE,
+		 "time: %-8.5f saddr: %-2d sport: %-2d daddr: %-2d dport:"
+		 " %-2d maxseq: %-4d hiack: %-4d seqno: %-4d cwnd: %-6.3f"
+		 " ssthresh: %-3d dupacks: %-2d rtt: %-6.3f srtt: %-6.3f"
+		 " rttvar: %-6.3f bkoff: %-d\n", curtime, addr(), port(),
+		 daddr(), dport(), int(maxseq_), int(highest_ack_),
+		 int(t_seqno_), double(cwnd_), int(ssthresh_),
+		 int(dupacks_), int(t_rtt_)*tcp_tick_, 
+		 (int(t_srtt_) >> T_SRTT_BITS)*tcp_tick_, 
+		 int(t_rttvar_)*tcp_tick_/4.0, int(t_backoff_)); 
+	(void)Tcl_Write(channel_, wrk, -1);
 }
 
 /* Print out just the variable that is modified */
 void
 TcpAgent::traceVar(TracedVar* v) 
 {
+	if (!channel_)
+		return;
+
 	double curtime;
 	Scheduler& s = Scheduler::instance();
-	char wrk[500];
-	int n;
+	char wrk[TCP_WRK_SIZE];
 
 	curtime = &s ? s.clock() : 0;
-	if (!strcmp(v->name(), "cwnd_") || !strcmp(v->name(), "maxrto_")) 
-		sprintf(wrk,"%-8.5f %-2d %-2d %-2d %-2d %s %-6.3f",
-			curtime, addr(), port(), daddr(), dport(),
-			v->name(), double(*((TracedDouble*) v))); 
-	else if (!strcmp(v->name(), "minrto_")) 
-		sprintf(wrk,"%-8.5f %-2d %-2d %-2d %-2d %s %-6.3f",
-			curtime, addr(), port(), daddr(), dport(),
-			v->name(), double(*((TracedDouble*) v))); 
-	else if (!strcmp(v->name(), "rtt_"))
-		sprintf(wrk,"%-8.5f %-2d %-2d %-2d %-2d %s %-6.3f",
-			curtime, addr(), port(), daddr(), dport(),
-			v->name(), int(*((TracedInt*) v))*tcp_tick_); 
-	else if (!strcmp(v->name(), "srtt_")) 
-		sprintf(wrk,"%-8.5f %-2d %-2d %-2d %-2d %s %-6.3f",
-			curtime, addr(), port(), daddr(), dport(),
-			v->name(), 
-			(int(*((TracedInt*) v)) >> T_SRTT_BITS)*tcp_tick_); 
-	else if (!strcmp(v->name(), "rttvar_"))
-		sprintf(wrk,"%-8.5f %-2d %-2d %-2d %-2d %s %-6.3f",
-			curtime, addr(), port(), daddr(), dport(),
-			v->name(), 
-			int(*((TracedInt*) v))*tcp_tick_/4.0); 
+
+	// XXX comparing addresses is faster than comparing names
+	if (v == &cwnd_)
+		snprintf(wrk, TCP_WRK_SIZE,
+			 "%-8.5f %-2d %-2d %-2d %-2d %s %-6.3f\n",
+			 curtime, addr(), port(), daddr(), dport(),
+			 v->name(), double(*((TracedDouble*) v))); 
+ 	else if (v == &t_rtt_)
+		snprintf(wrk, TCP_WRK_SIZE,
+			 "%-8.5f %-2d %-2d %-2d %-2d %s %-6.3f\n",
+			 curtime, addr(), port(), daddr(), dport(),
+			 v->name(), int(*((TracedInt*) v))*tcp_tick_); 
+	else if (v == &t_srtt_)
+		snprintf(wrk, TCP_WRK_SIZE,
+			 "%-8.5f %-2d %-2d %-2d %-2d %s %-6.3f\n",
+			 curtime, addr(), port(), daddr(), dport(),
+			 v->name(), 
+			 (int(*((TracedInt*) v)) >> T_SRTT_BITS)*tcp_tick_); 
+	else if (v == &t_rttvar_)
+		snprintf(wrk, TCP_WRK_SIZE,
+			 "%-8.5f %-2d %-2d %-2d %-2d %s %-6.3f\n",
+			 curtime, addr(), port(), daddr(), dport(),
+			 v->name(), 
+			 int(*((TracedInt*) v))*tcp_tick_/4.0); 
 	else
-		sprintf(wrk,"%-8.5f %-2d %-2d %-2d %-2d %s %d",
-			curtime, addr(), port(), daddr(), dport(),
-			v->name(), int(*((TracedInt*) v))); 
-	n = strlen(wrk);
-	wrk[n] = '\n';
-	wrk[n+1] = 0;
-	if (channel_)
-		(void)Tcl_Write(channel_, wrk, n+1);
-	wrk[n] = 0;
-	return;
+		snprintf(wrk, TCP_WRK_SIZE,
+			 "%-8.5f %-2d %-2d %-2d %-2d %s %d\n",
+			 curtime, addr(), port(), daddr(), dport(),
+			 v->name(), int(*((TracedInt*) v))); 
+
+	(void)Tcl_Write(channel_, wrk, -1);
 }
 
 void
@@ -420,6 +455,7 @@ TcpAgent::reset()
 	maxseq_ = -1;
 	last_ack_ = -1;
 	highest_ack_ = -1;
+	//highest_ack_ = 1;
 	ssthresh_ = int(wnd_);
 	if (max_ssthresh_ > 0 && max_ssthresh_ < ssthresh_) 
 		ssthresh_ = max_ssthresh_;
@@ -444,13 +480,25 @@ TcpAgent::reset()
 	nrexmitpack_ = 0;
 	necnresponses_ = 0;
 	ncwndcuts_ = 0;
-
-        cwnd_last_ = 0.0;
+	ncwndcuts1_ = 0;
 
 	if (control_increase_) {
 		prev_highest_ack_ = highest_ack_ ; 
 	}
 
+	if (wnd_option_ == 8) {
+		// HighSpeed TCP
+		hstcp_.low_p = 1.5/(low_window_*low_window_);
+		double highLowWin = log(high_window_)-log(low_window_);
+		double highLowP = log(high_p_) - log(hstcp_.low_p);
+		hstcp_.dec1 = 
+		   0.5 - log(low_window_) * (high_decrease_ - 0.5)/highLowWin;
+		hstcp_.dec2 = (high_decrease_ - 0.5)/highLowWin;
+        	hstcp_.p1 = 
+		  log(hstcp_.low_p) - log(low_window_) * highLowP/highLowWin;
+		hstcp_.p2 = highLowP/highLowWin;
+	}
+
 	if (QOption_) {
 		int now = (int)(Scheduler::instance().clock()/tcp_tick_ + 0.5);
 		T_last = now ; 
@@ -480,11 +528,12 @@ double TcpAgent::rtt_timeout()
 	if (rfc2988_) {
 	// Correction from Tom Kelly to be RFC2988-compliant, by
 	// clamping minrto_ before applying t_backoff_.
-		if (t_rtxcur_ < minrto_)
+		if (t_rtxcur_ < minrto_ && !use_rtt_)
 			timeout = minrto_ * t_backoff_;
 		else
 			timeout = t_rtxcur_ * t_backoff_;
 	} else {
+		// only of interest for backwards compatibility
 		timeout = t_rtxcur_ * t_backoff_;
 		if (timeout < minrto_)
 			timeout = minrto_;
@@ -498,9 +547,12 @@ double TcpAgent::rtt_timeout()
 			fprintf(stderr, "TcpAgent: negative RTO!  (%f)\n",
 				timeout);
 			exit(1);
-		}
-		timeout = 2.0 * tcp_tick_;
+		} else if (use_rtt_ && timeout < tcp_tick_)
+			timeout = tcp_tick_;
+		else
+			timeout = 2.0 * tcp_tick_;
 	}
+	use_rtt_ = 0;
 	return (timeout);
 }
 
@@ -519,10 +571,13 @@ void TcpAgent::rtt_update(double tao)
 	}
 	if (t_rtt_ < 1)
 		t_rtt_ = 1;
-
 	//
-	// srtt has 3 bits to the right of the binary point
-	// rttvar has 2
+	// t_srtt_ has 3 bits to the right of the binary point
+	// t_rttvar_ has 2
+        // Thus "t_srtt_ >> T_SRTT_BITS" is the actual srtt, 
+  	//   and "t_srtt_" is 8*srtt.
+	// Similarly, "t_rttvar_ >> T_RTTVAR_BITS" is the actual rttvar,
+	//   and "t_rttvar_" is 4*rttvar.
 	//
         if (t_srtt_ != 0) {
 		register short delta;
@@ -551,8 +606,10 @@ void TcpAgent::rtt_update(double tao)
 
 void TcpAgent::rtt_backoff()
 {
-	if (t_backoff_ < 64)
-		t_backoff_ <<= 1;
+	if (t_backoff_ < 64 || rfc2988_)
+        	t_backoff_ <<= 1;
+        // RFC2988 allows a maximum for the backed-off RTO of 60 seconds.
+        // This is applied by maxrto_.
 
 	if (t_backoff_ > 8) {
 		/*
@@ -593,6 +650,36 @@ void TcpAgent::output(int seqno, int reason)
 	int databytes = hdr_cmn::access(p)->size();
 	tcph->seqno() = seqno;
 	tcph->ts() = Scheduler::instance().clock();
+	int is_retransmit = (seqno < maxseq_);
+ 
+	// Mark packet for diagnosis purposes if we are in Quick-Start Phase
+	if (qs_approved_) {
+		hf->qs() = 1;
+	}
+ 
+        // store timestamps, with bugfix_ts_.  From Andrei Gurtov. 
+	// (A real TCP would use scoreboard for this.)
+        if (bugfix_ts_ && tss==NULL) {
+                tss = (double*) calloc(tss_size_, sizeof(double));
+                if (tss==NULL) exit(1);
+        }
+        //dynamically grow the timestamp array if it's getting full
+        if (bugfix_ts_ && window() > tss_size_* 0.9) {
+                double *ntss;
+                ntss = (double*) calloc(tss_size_*2, sizeof(double));
+                printf("resizing timestamp table\n");
+                if (ntss == NULL) exit(1);
+                for (int i=0; i<tss_size_; i++)
+                        ntss[(highest_ack_ + i) % (tss_size_ * 2)] =
+                                tss[(highest_ack_ + i) % tss_size_];
+                free(tss);
+                tss_size_ *= 2;
+                tss = ntss;
+        }
+ 
+        if (tss!=NULL)
+                tss[seqno % tss_size_] = tcph->ts(); 
+
 	tcph->ts_echo() = ts_peer_;
 	tcph->reason() = reason;
 	tcph->last_rtt() = int(int(t_rtt_)*tcp_tick_*1000);
@@ -600,7 +687,7 @@ void TcpAgent::output(int seqno, int reason)
 	if (ecn_) {
 		hf->ect() = 1;	// ECN-capable transport
 	}
-	if (cong_action_) {
+	if (cong_action_ && (!is_retransmit || SetCWRonRetransmit_)) {
 		hf->cong_action() = TRUE;  // Congestion action.
 		cong_action_ = FALSE;
         }
@@ -618,13 +705,37 @@ void TcpAgent::output(int seqno, int reason)
 		}
 		if (qs_enabled_) {
 			hdr_qs *qsh = hdr_qs::access(p);
-		    	if (rate_request_ > 0) {
+
+			// dataout is kilobytes queued for sending
+			int dataout = (curseq_ - maxseq_ - 1) * (size_ + headersize()) / 1024;
+			int qs_rr = rate_request_;
+			if (qs_request_mode_ == 1 && qs_rtt_ > 0) {
+				// PS: Avoid making unnecessary QS requests
+				// use a rough estimation of RTT in qs_rtt_
+				// to calculate the desired rate from dataout.
+				// printf("dataout %d qs_rr %d qs_rtt_ %d\n",
+				//	dataout, qs_rr, qs_rtt_);
+				if (dataout * 1000 / qs_rtt_ < qs_rr) {
+					qs_rr = dataout * 1000 / qs_rtt_;
+				}
+				// printf("request %d\n", qs_rr);
+				// qs_thresh_ is minimum number of unsent
+				// segments needed to activate QS request
+				// printf("curseq_ %d maxseq_ %d qs_thresh_ %d\n",
+				//	 int(curseq_), int(maxseq_), qs_thresh_);
+				if ((curseq_ - maxseq_ - 1) < qs_thresh_) {
+					qs_rr = 0;
+				}
+			} 
+
+		    	if (qs_rr > 0) {
+				if (print_request_) 
+					printf("QS request (before encoding): %d KBps\n", qs_rr);
 				// QuickStart code from Srikanth Sundarrajan.
 				qsh->flag() = QS_REQUEST;
-				Random::seed_heuristically();
 				qsh->ttl() = Random::integer(256);
 				ttl_diff_ = (iph->ttl() - qsh->ttl()) % 256;
-				qsh->rate() = rate_request_;
+				qsh->rate() = hdr_qs::Bps_to_rate(qs_rr * 1024);
 				qs_requested_ = 1;
 		    	} else {
 				qsh->flag() = QS_DISABLE;
@@ -748,9 +859,29 @@ int TcpAgent::command(int argc, const char*const* argv)
 	return (Agent::command(argc, argv));
 }
 
+/*
+ * Returns the window size adjusted to allow <num> segments past recovery
+ * point to be transmitted on next ack.
+ */
+int TcpAgent::force_wnd(int num)
+{
+	return recover_ + num - (int)highest_ack_;
+}
+
 int TcpAgent::window()
 {
-	return (cwnd_ < wnd_ ? (int)cwnd_ : (int)wnd_);
+        /*
+         * If F-RTO is enabled and first ack has come in, temporarily open
+         * window for sending two segments.
+	 * The F-RTO code is from Pasi Sarolahti.  F-RTO is an algorithm
+	 * for detecting spurious retransmission timeouts.
+         */
+        if (frto_ == 2) {
+                return (force_wnd(2) < wnd_ ?
+                        force_wnd(2) : (int)wnd_);
+        } else {
+		return (cwnd_ < wnd_ ? (int)cwnd_ : (int)wnd_);
+        }
 }
 
 double TcpAgent::windowd()
@@ -777,7 +908,7 @@ void TcpAgent::send_much(int force, int reason, int maxburst)
 	if (burstsnd_timer_.status() == TIMER_PENDING)
 		return;
 	while (t_seqno_ <= highest_ack_ + win && t_seqno_ < curseq_) {
-		if (overhead_ == 0 || force) {
+		if (overhead_ == 0 || force || qs_approved_) {
 			output(t_seqno_, reason);
 			npackets++;
 			if (QOption_)
@@ -786,7 +917,11 @@ void TcpAgent::send_much(int force, int reason, int maxburst)
 			if (qs_approved_ == 1) {
 				// delay = effective RTT / window
 				double delay = (double) t_rtt_ * tcp_tick_ / win;
-				delsnd_timer_.resched(delay);
+				if (overhead_) { 
+					delsnd_timer_.resched(delay + Random::uniform(overhead_));
+				} else {
+					delsnd_timer_.resched(delay);
+				}
 				return;
 			}
 		} else if (!(delsnd_timer_.status() == TIMER_PENDING)) {
@@ -867,15 +1002,24 @@ double TcpAgent::linear(double x, double x_1, double y_1, double x_2, double y_2
 
 /*
  * Limited Slow-Start for large congestion windows.
- * This is only used when max_ssthresh_ is non-zero.
+ * This should only be called when max_ssthresh_ is non-zero.
  */
-double TcpAgent::limited_slow_start(double cwnd, double max_ssthresh, double increment)
+double TcpAgent::limited_slow_start(double cwnd, int max_ssthresh, double increment)
 {
-	int round = int(cwnd / (double(max_ssthresh)/2.0));
-	double increment1 = 1.0/(double(round)); 
-	if (increment < increment1)
-		increment = increment1;
-	return increment;
+        if (max_ssthresh <= 0) {
+	  	return increment;
+	} else {
+                double increment1 = 0.0;
+		int round = int(cwnd / (double(max_ssthresh)/2.0));
+		if (round > 0) {
+		  	increment1 = 1.0/double(round); 
+		} 
+		if (increment < increment1) {
+		  	return increment1;
+		} else {
+		  	return increment;
+		}
+	}
 }
 
 /*
@@ -891,24 +1035,16 @@ int TcpAgent::numdupacks(double cwnd)
 	}
 }
 
-/*
- * Calculating the packet drop rate p for highspeed TCP.
- */
-double TcpAgent::compute_p()
-{
-	double p;
-	double low_p = 1.5/(low_window_*low_window_);
-	p = exp(linear(log(cwnd_), log(low_window_), log(low_p), log(high_window_), log(high_p_)));
-	return p;
-}
-
 /*
  * Calculating the decrease parameter for highspeed TCP.
  */
 double TcpAgent::decrease_param()
 {
 	double decrease;
-	decrease = linear(log(cwnd_), log(low_window_), 0.5, log(high_window_), high_decrease_);
+	// OLD:
+	// decrease = linear(log(cwnd_), log(low_window_), 0.5, log(high_window_), high_decrease_);
+	// NEW (but equivalent):
+        decrease = hstcp_.dec1 + log(cwnd_) * hstcp_.dec2;  
 	return decrease;
 }
 
@@ -934,20 +1070,28 @@ double TcpAgent::increase_param()
        if (cwnd_ <= low_window_) { 
 		answer = 1 / cwnd_;
        		return answer; 
-       } else if (cwnd_ >= cwnd_last_ && cwnd_ < cwnd_frac_ * cwnd_last_ ) {
-		answer = increase_last_ / cwnd_;
-                return answer;
+       } else if (cwnd_ >= hstcp_.cwnd_last_ && 
+	      cwnd_ < hstcp_.cwnd_last_ + cwnd_range_) {
+	      // cwnd_range_ can be set to 0 to be disabled,
+	      //  or can be set from 1 to 100 
+       		answer = hstcp_.increase_last_ / cwnd_;
+              	return answer;
        } else { 
-		p = compute_p();
-		decrease = decrease_param();
-		increase = (cwnd_ * cwnd_ *2.0* decrease * p)/(2.0 - decrease); 
-		//      double max_increase = 157.8;
+		// OLD:
+ 		// p = exp(linear(log(cwnd_), log(low_window_), log(hstcp_.low_p), log(high_window_), log(high_p_)));
+		// NEW, but equivalent:
+        	p = exp(hstcp_.p1 + log(cwnd_) * hstcp_.p2);  
+        	decrease = decrease_param();
+		// OLD:
+		// increase = cwnd_*cwnd_*p *(2.0*decrease)/(2.0 - decrease); 
+		// NEW, but equivalent:
+		increase = cwnd_ * cwnd_ * p /(1/decrease - 0.5);
 		//	if (increase > max_increase) { 
 		//		increase = max_increase;
 		//	} 
 		answer = increase / cwnd_;
-		cwnd_last_ = cwnd_;
-		increase_last_ = increase;
+		hstcp_.cwnd_last_ = cwnd_;
+		hstcp_.increase_last_ = increase;
        		return answer;
 	}       
 }
@@ -987,9 +1131,15 @@ void TcpAgent::opencwnd()
 		case 2:
 			/* These are window increase algorithms
 			 * for experimental purposes only. */
+			/* This is the Constant-Rate increase algorithm 
+                         *  from the 1991 paper by S. Floyd on "Connections  
+			 *  with Multiple Congested Gateways". 
+			 *  The window is increased by roughly 
+			 *  wnd_const_*RTT^2 packets per round-trip time.  */
 			f = (t_srtt_ >> T_SRTT_BITS) * tcp_tick_;
 			f *= f;
 			f *= wnd_const_;
+			/* f = wnd_const_ * RTT^2 */
 			f += fcnt_;
 			if (f > cwnd_) {
 				fcnt_ = 0;
@@ -999,6 +1149,9 @@ void TcpAgent::opencwnd()
 			break;
 
 		case 3:
+			/* The window is increased by roughly 
+			 *  awnd_^2 * wnd_const_ packets per RTT,
+			 *  for awnd_ the average congestion window. */
 			f = awnd_;
 			f *= f;
 			f *= wnd_const_;
@@ -1011,6 +1164,9 @@ void TcpAgent::opencwnd()
 			break;
 
                 case 4:
+			/* The window is increased by roughly 
+			 *  awnd_ * wnd_const_ packets per RTT,
+			 *  for awnd_ the average congestion window. */
                         f = awnd_;
                         f *= wnd_const_;
                         f += fcnt_;
@@ -1021,6 +1177,10 @@ void TcpAgent::opencwnd()
                                 fcnt_ = f;
                         break;
 		case 5:
+			/* The window is increased by roughly wnd_const_*RTT 
+			 *  packets per round-trip time, as discussed in
+			 *  the 1992 paper by S. Floyd on "On Traffic 
+			 *  Phase Effects in Packet-Switched Gateways". */
                         f = (t_srtt_ >> T_SRTT_BITS) * tcp_tick_;
                         f *= wnd_const_;
                         f += fcnt_;
@@ -1035,7 +1195,7 @@ void TcpAgent::opencwnd()
                         cwnd_ += increase_num_ / (cwnd_*pow(cwnd_,k_parameter_));                
                         break; 
  		case 8: 
-			/* high-speed TCP */
+			/* high-speed TCP, RFC 3649 */
 			increment = increase_param();
 			if ((last_cwnd_action_ == 0 ||
 			  last_cwnd_action_ == CWND_ACTION_TIMEOUT) 
@@ -1066,7 +1226,10 @@ TcpAgent::slowdown(int how)
 	double decrease;  /* added for highspeed - sylvia */
 	double win, halfwin, decreasewin;
 	int slowstart = 0;
-	++ncwndcuts_; 
+	++ncwndcuts_;
+	if (!(how & TCP_IDLE) && !(how & NO_OUTSTANDING_DATA)){
+		++ncwndcuts1_; 
+	}
 	// we are in slowstart for sure if cwnd < ssthresh
 	if (cwnd_ < ssthresh_) 
 		slowstart = 1;
@@ -1166,8 +1329,6 @@ TcpAgent::slowdown(int how)
 	
 }
 
-
-
 /*
  * Process a packet that acks previously unacknowleged data.
  */
@@ -1195,9 +1356,20 @@ void TcpAgent::newack(Packet* pkt)
 	 */	
 	hdr_flags *fh = hdr_flags::access(pkt);
 	if (!fh->no_ts_) {
-		if (ts_option_)
+		if (ts_option_) {
+			ts_echo_=tcph->ts_echo();
 			rtt_update(now - tcph->ts_echo());
-
+			if (ts_resetRTO_ && (!ect_ || !ecn_backoff_ ||
+			    !hdr_flags::access(pkt)->ecnecho())) { 
+				// From Andrei Gurtov
+				/* 
+				 * Don't end backoff if still in ECN-Echo with
+			 	 * a congestion window of 1 packet. 
+				 */
+				t_backoff_ = 1;
+				ecn_backoff_ = 0;
+			}
+		}
 		if (rtt_active_ && tcph->seqno() >= rtt_seq_) {
 			if (!ect_ || !ecn_backoff_ || 
 				!hdr_flags::access(pkt)->ecnecho()) {
@@ -1261,6 +1433,11 @@ int TcpAgent::network_limited() {
 void TcpAgent::recv_newack_helper(Packet *pkt) {
 	//hdr_tcp *tcph = hdr_tcp::access(pkt);
 	newack(pkt);
+        if (qs_window_ && highest_ack_ >= qs_window_) {
+                // All segments in the QS window have been acknowledged.
+                // We can exit the Quick-Start phase.
+                qs_window_ = 0;
+        }
 	if (!ect_ || !hdr_flags::access(pkt)->ecnecho() ||
 		(old_ecn_ && ecn_burst_)) {
 		/* If "old_ecn", this is not the first ACK carrying ECN-Echo
@@ -1292,6 +1469,33 @@ void TcpAgent::recv_newack_helper(Packet *pkt) {
 	if (QOption_ && curseq_ == highest_ack_ +1) {
 		cancel_rtx_timer();
 	}
+	if (frto_ == 1) {
+		/*
+		 * New ack after RTO. If F-RTO is enabled, try to transmit new
+		 * previously unsent segments.
+		 * If there are no new data or receiver window limits the
+		 * transmission, revert to traditional recovery.
+		 */
+		if (recover_ + 1 >= highest_ack_ + wnd_ ||
+		    recover_ + 1 >= curseq_) {
+			frto_ = 0;
+ 		} else if (highest_ack_ == recover_) {
+ 			/*
+ 			 * F-RTO step 2a) RTO retransmission fixes whole
+			 * window => cancel F-RTO
+ 			 */
+ 			frto_ = 0;
+		} else {
+			t_seqno_ = recover_ + 1;
+			frto_ = 2;
+		}
+	} else if (frto_ == 2) {
+		/*
+		 * Second new ack after RTO. If F-RTO is enabled, RTO can be
+		 * declared spurious
+		 */
+		spurious_timeout();
+	}
 }
 
 /*
@@ -1300,6 +1504,11 @@ void TcpAgent::recv_newack_helper(Packet *pkt) {
 double
 TcpAgent::initial_window()
 {
+        // If Quick-Start Request was approved, use that as a basis for
+        // initial window
+        if (qs_cwnd_) {
+                return (qs_cwnd_);
+        }
 	//
 	// init_option = 1: static iw of wnd_init_
 	//
@@ -1364,7 +1573,11 @@ void
 TcpAgent::dupack_action()
 {
 	int recovered = (highest_ack_ > recover_);
-	if (recovered || (!bug_fix_ && !ecn_)) {
+	if (recovered || (!bug_fix_ && !ecn_) || 
+		(bugfix_ss_ && highest_ack_ == 0)) {
+		// (highest_ack_ == 0) added to allow Fast Retransmit
+		//  when the first data packet is dropped.
+		//  Bug report from Mark Allman.
 		goto tahoe_action;
 	}
 
@@ -1385,12 +1598,13 @@ TcpAgent::dupack_action()
 	}
 
 tahoe_action:
-	// we are now going to fast-retransmit and willtrace that event
-	trace_event("FAST_RETX");
-
-	recover_ = maxseq_;
-	last_cwnd_action_ = CWND_ACTION_DUPACK;
-	slowdown(CLOSE_SSTHRESH_HALF|CLOSE_CWND_ONE);
+        recover_ = maxseq_;
+        if (!lossQuickStart()) {
+		// we are now going to fast-retransmit and willtrace that event
+		trace_event("FAST_RETX");
+		last_cwnd_action_ = CWND_ACTION_DUPACK;
+		slowdown(CLOSE_SSTHRESH_HALF|CLOSE_CWND_ONE);
+	}
 	reset_rtx_timer(0,0);
 	return;
 }
@@ -1402,6 +1616,8 @@ tahoe_action:
 void TcpAgent::endQuickStart()
 {
 	qs_approved_ = 0;
+        qs_cwnd_ = 0;
+        qs_window_ = maxseq_;
 	int new_cwnd = maxseq_ - last_ack_;
 	if (new_cwnd > 1 && new_cwnd < cwnd_) {
 	 	cwnd_ = new_cwnd;
@@ -1424,20 +1640,120 @@ void TcpAgent::processQuickStart(Packet *pkt)
 	qs_approved_ = 0;
 	if (qsh->flag() == QS_RESPONSE && qsh->ttl() == ttl_diff_ && 
             qsh->rate() > 0) {
-                app_rate = (int) (qsh->rate() * (now - tcph->ts_echo())) ;
-		printf("Quick Start approved, rate %d, window %d\n", 
-				     qsh->rate(), app_rate);
+                app_rate = (int) ((hdr_qs::rate_to_Bps(qsh->rate()) *
+                      (now - tcph->ts_echo())) / (size_ + headersize()));
+		if (print_request_) {
+		  double num1 = hdr_qs::rate_to_Bps(qsh->rate());
+		  double time = now - tcph->ts_echo();
+		  int size = size_ + headersize();
+		  printf("Quick Start request, rate: %g Bps, encoded rate: %d\n", 
+		     num1, qsh->rate());
+		  printf("Quick Start request, window %d rtt: %4.2f pktsize: %d\n",
+		     app_rate, time, size);
+		}
                 if (app_rate > initial_window()) {
-                        wnd_init_option_ = 1;
-                        wnd_init_ = app_rate;
+			qs_cwnd_ = app_rate;
                         qs_approved_ = 1;
                 }
         } else { // Quick Start rejected
+#ifdef QS_DEBUG
                 printf("Quick Start rejected\n");
+#endif
         }
 
 }
 
+
+
+/*
+ * ACK has been received, hook from recv()
+ */
+void TcpAgent::recv_frto_helper(Packet *pkt)
+{
+	hdr_tcp *tcph = hdr_tcp::access(pkt);
+	if (tcph->seqno() == last_ack_ && frto_ != 0) {
+		/*
+		 * Duplicate ACK while in F-RTO indicates that the
+		 * timeout was valid. Go to slow start retransmissions.
+		 */
+		t_seqno_ = highest_ack_ + 1;
+		cwnd_ = frto_;
+		frto_ = 0;
+
+		// Must zero dupacks (in order to trigger send_much at recv)
+		// dupacks is increased in recv after exiting this function
+		dupacks_ = -1;
+	}
+}
+
+
+/*
+ * A spurious timeout has been detected. Do appropriate actions.
+ */
+void TcpAgent::spurious_timeout()
+{
+	frto_ = 0;
+
+	switch (spurious_response_) {
+	case 1:
+	default:
+		/*
+		 * Full revert of congestion window
+		 * (FlightSize before last acknowledgment)
+		 */
+		cwnd_ = t_seqno_ - prev_highest_ack_;
+		break;
+ 
+	case 2:
+		/*
+		 * cwnd = reduced ssthresh (approx. half of the earlier pipe)
+		 */
+		cwnd_ = ssthresh_; break;
+	case 3:
+		/*
+		 * slow start, but without retransmissions
+		 */
+		cwnd_ = 1; break;
+	}
+
+	/*
+	 * Revert ssthresh to size before retransmission timeout
+	 */
+	ssthresh_ = pipe_prev_;
+
+	/* If timeout was spurious, bugfix is not needed */
+	recover_ = highest_ack_ - 1;
+}
+
+
+/*
+ * Loss occurred in Quick-Start window.
+ * If Quick-Start is enabled, packet loss in the QS phase should
+ * trigger slow start instead of the regular fast retransmit,
+ * see [draft-amit-quick-start-03.txt] (to appear).
+ * We use variable tcp_qs_recovery_ to toggle this behaviour on and off.
+ * If tcp_qs_recovery_ is true, initiate slow start to probe for
+ * a correct window size.
+ *
+ * Return value: non-zero if Quick-Start specific loss recovery took place
+ */
+int TcpAgent::lossQuickStart()
+{
+       if (qs_window_ && tcp_qs_recovery_) {
+                //recover_ = maxseq_;
+                //reset_rtx_timer(1,0);
+                slowdown(CLOSE_CWND_INIT);
+		// reset ssthresh to half of W-D/2?
+                qs_window_ = 0;
+                output(last_ack_ + 1, TCP_REASON_DUPACK);
+                return 1;
+       }
+       return 0;
+}
+
+
+
+
 /*
  * main reception path - should only see acks, otherwise the
  * network connections are misconfigured
@@ -1445,6 +1761,7 @@ void TcpAgent::processQuickStart(Packet *pkt)
 void TcpAgent::recv(Packet *pkt, Handler*)
 {
 	hdr_tcp *tcph = hdr_tcp::access(pkt);
+	int valid_ack = 0;
 	if (qs_approved_ == 1 && tcph->seqno() > last_ack_) 
 		endQuickStart();
 	if (qs_requested_ == 1)
@@ -1469,6 +1786,7 @@ void TcpAgent::recv(Packet *pkt, Handler*)
 	if (ecnecho && ecn_)
 		ecn(tcph->seqno());
 	recv_helper(pkt);
+	recv_frto_helper(pkt);
 	/* grow cwnd and check if the connection is done */ 
 	if (tcph->seqno() > last_ack_) {
 		recv_newack_helper(pkt);
@@ -1490,11 +1808,15 @@ void TcpAgent::recv(Packet *pkt, Handler*)
 	if (QOption_ && EnblRTTCtr_)
 		process_qoption_after_ack (tcph->seqno());
 
+	if (tcph->seqno() >= last_ack_)  
+		// Check if ACK is valid.  Suggestion by Mark Allman. 
+		valid_ack = 1;
 	Packet::free(pkt);
 	/*
 	 * Try to send more data.
 	 */
-	send_much(0, 0, maxburst_);
+	if (valid_ack || aggressive_maxburst_)
+		send_much(0, 0, maxburst_);
 }
 
 /*
@@ -1521,6 +1843,11 @@ void TcpAgent::timeout(int tno)
 		// There has been a timeout - will trace this event
 		trace_event("TIMEOUT");
 
+		frto_ = 0;
+		// Set pipe_prev as per Eifel Response
+		pipe_prev_ = (window() > ssthresh_) ?
+			window() : (int)ssthresh_;
+
 	        if (cwnd_ < 1) cwnd_ = 1;
 		if (qs_approved_ == 1) qs_approved_ = 0;
 		if (highest_ack_ == maxseq_ && !slow_start_restart_) {
@@ -1539,12 +1866,20 @@ void TcpAgent::timeout(int tno)
 				 * initial windows. 
 				 */
 				wnd_init_option_ = 1;
+                        else if ((highest_ack_ == -1) &&
+                                (wnd_init_option_ == 1) && (wnd_init_ > 1)
+				&& bugfix_ss_)
+                                /*
+                                 * First packet dropped, so don't use larger
+                                 * initial windows.  Bugfix from Mark Allman.
+                                 */
+                                wnd_init_ = 1;
 			if (highest_ack_ == maxseq_ && restart_bugfix_)
 			       /* 
 				* if there is no outstanding data, don't cut 
 				* down ssthresh_.
 				*/
-				slowdown(CLOSE_CWND_ONE);
+				slowdown(CLOSE_CWND_ONE|NO_OUTSTANDING_DATA);
 			else if (highest_ack_ < recover_ &&
 			  last_cwnd_action_ == CWND_ACTION_ECN) {
 			       /*
@@ -1552,11 +1887,17 @@ void TcpAgent::timeout(int tno)
 				* don't cut down ssthresh_.
 				*/
 				slowdown(CLOSE_CWND_ONE);
+				if (frto_enabled_ || sfrto_enabled_) {
+					frto_ = 1;
+				}
 			}
 			else {
 				++nrexmit_;
 				last_cwnd_action_ = CWND_ACTION_TIMEOUT;
 				slowdown(CLOSE_SSTHRESH_HALF|CLOSE_CWND_RESTART);
+				if (frto_enabled_ || sfrto_enabled_) {
+					frto_ = 1;
+				}
 			}
 		}
 		/* if there is no outstanding data, don't back off rtx timer */
@@ -1708,9 +2049,9 @@ void TcpAgent::process_qoption_after_send ()
 	if (!EnblRTTCtr_) {
 		if (tcp_now - T_last >= rto) {
 			// The sender has been idle.
-		 	slowdown(THREE_QUARTER_SSTHRESH) ;
+		 	slowdown(THREE_QUARTER_SSTHRESH|TCP_IDLE) ;
 			for (int i = 0 ; i < (tcp_now - T_last)/rto; i ++) {
-				slowdown(CWND_HALF_WITH_MIN);
+				slowdown(CWND_HALF_WITH_MIN|TCP_IDLE);
 			}
 			T_prev = tcp_now ;
 			W_used = 0 ;
@@ -1727,8 +2068,8 @@ void TcpAgent::process_qoption_after_send ()
 				W_used = tmp ;
 			if (tcp_now - T_prev >= rto) {
 				// The sender has been application-limited.
-				slowdown(THREE_QUARTER_SSTHRESH);
-				slowdown(CLOSE_CWND_HALF_WAY);
+				slowdown(THREE_QUARTER_SSTHRESH|TCP_IDLE);
+				slowdown(CLOSE_CWND_HALF_WAY|TCP_IDLE);
 				T_prev = tcp_now ;
 				W_used = 0 ;
 			}
@@ -1759,9 +2100,9 @@ TcpAgent::rtt_counting()
 		RTTs = RTTs - Backoffs ; 
 		Backoffs = 0 ; 
 		if (RTTs > 0) {
-			slowdown(THREE_QUARTER_SSTHRESH) ;
+			slowdown(THREE_QUARTER_SSTHRESH|TCP_IDLE) ;
 			for (int i = 0 ; i < RTTs ; i ++) {
-				slowdown(CWND_HALF_WITH_MIN);
+				slowdown(CWND_HALF_WITH_MIN|TCP_IDLE);
 				RTT_prev = RTT_count ; 
 				W_used = 0 ;
 			}
@@ -1790,8 +2131,8 @@ TcpAgent::rtt_counting()
 			W_used = tmp ;
 		if (RTT_count - RTT_prev >= 2) {
 			// The sender has been application-limited.
-			slowdown(THREE_QUARTER_SSTHRESH) ;
-			slowdown(CLOSE_CWND_HALF_WAY);
+			slowdown(THREE_QUARTER_SSTHRESH|TCP_IDLE) ;
+			slowdown(CLOSE_CWND_HALF_WAY|TCP_IDLE);
 			RTT_prev = RTT_count ; 
 			Backoffs ++ ; 
 			W_used = 0;
