@@ -34,23 +34,31 @@
  * Ported from CMU/Monarch's code, appropriate copyright applies.
  * nov'98 -Padma.
  *
- * $Header: /nfs/jade/vint/CVSROOT/ns-2/trace/cmu-trace.cc,v 1.72 2003/02/22 03:53:35 buchheim Exp $
+ * $Header: /cvsroot/nsnam/ns-2/trace/cmu-trace.cc,v 1.89 2006/03/21 22:31:32 liyuan Exp $
  */
 
 #include <packet.h>
 #include <ip.h>
 #include <tcp.h>
+#include <sctp.h>
 #include <rtp.h>
 #include <arp.h>
 #include <dsr/hdr_sr.h>	// DSR
 #include <mac.h>
 #include <mac-802_11.h>
+#include <smac.h>
 #include <address.h>
 #include <tora/tora_packet.h> //TORA
 #include <imep/imep_spec.h>         // IMEP
 #include <aodv/aodv_packet.h> //AODV
 #include <cmu-trace.h>
 #include <mobilenode.h>
+#include <simulator.h>
+//<zheng: add for 802.15.4>
+#include "wpan/p802_15_4pkt.h"
+#include "wpan/p802_15_4trace.h"
+#include "wpan/p802_15_4nam.h"
+//</zheng: add for 802.15.4>
 
 #include "diffusion/diff_header.h" // DIFFUSION -- Chalermek
 
@@ -68,11 +76,11 @@ public:
 } cmutrace_class;
 
 
-
+//<zheng: ns 2.27 removed the following part, but we need it to control the broadcast radius>
 double CMUTrace::bradius = 0.0;
 double CMUTrace::radius_scaling_factor_ = 0.0;
 double CMUTrace::duration_scaling_factor_ = 0.0;
-
+//</zheng>
 
 CMUTrace::CMUTrace(const char *s, char t) : Trace(t)
 {
@@ -98,22 +106,34 @@ CMUTrace::CMUTrace(const char *s, char t) : Trace(t)
                 fprintf(stderr, "CMU Trace Initialized with invalid type\n");
                 exit(1);
         }
+// change wrt Mike's code
+//	assert(type_ == DROP || type_ == SEND || type_ == RECV);
+	assert(type_ == DROP || type_ == SEND || type_ == RECV
+               || ((type_ == EOT) && (tracetype == TR_MAC)));
+
 
-	assert(type_ == DROP || type_ == SEND || type_ == RECV);
 
 	newtrace_ = 0;
 	for (int i=0 ; i < MAX_NODE ; i++) 
 		nodeColor[i] = 3 ;
         node_ = 0;
-
 }
 
 void
-CMUTrace::format_mac(Packet *p, const char *why, int offset)
+CMUTrace::format_mac_common(Packet *p, const char *why, int offset)
 {
 	struct hdr_cmn *ch = HDR_CMN(p);
 	struct hdr_ip *ih = HDR_IP(p);
-	struct hdr_mac802_11 *mh = HDR_MAC802_11(p);
+	struct hdr_mac802_11 *mh;
+	struct hdr_smac *sh;
+	char mactype[SMALL_LEN];
+
+	strcpy(mactype, Simulator::instance().macType());
+	if (strcmp (mactype, "Mac/SMAC") == 0)
+		sh = HDR_SMAC(p);
+	else
+		mh = HDR_MAC802_11(p);
+	
 	double x = 0.0, y = 0.0, z = 0.0;
        
 	char op = (char) type_;
@@ -149,8 +169,8 @@ CMUTrace::format_mac(Packet *p, const char *why, int offset)
 
 		sprintf(pt_->buffer() + offset,
 			"%c "TIME_FORMAT" -s %d -d %d -p %s -k %3s -i %d "
-			"-N:loc {%.2f %.2f %.2f} -N:en %f "
-			"-M:dur %x -M:s %x -M:d %x -M:t %x ",
+			"-N:loc {%.2f %.2f %.2f} -N:en %f ",
+			
 			op,				// event type
 			Scheduler::instance().clock(),	// time
 			src_,				// this node
@@ -159,12 +179,14 @@ CMUTrace::format_mac(Packet *p, const char *why, int offset)
 			tracename,			// trace level
 			ch->uid(),			// event id
 			x, y, z,			// location
-			energy,				// energy
-			mh->dh_duration,		// MAC: duration
-			ETHER_ADDR(mh->dh_da),		// MAC: source
-			ETHER_ADDR(mh->dh_sa),		// MAC: destination
-			GET_ETHER_TYPE(mh->dh_body)	// MAC: type
-			);
+			energy);				// energy
+
+		offset = strlen(pt_->buffer());
+		if (strcmp (mactype, "Mac/SMAC") == 0) {
+			format_smac(p, offset);
+		} else {
+			format_mac(p, offset);
+		}
 		return;
 	}
 
@@ -195,28 +217,25 @@ CMUTrace::format_mac(Packet *p, const char *why, int offset)
 	    // mac layer extension
 
 	    offset = strlen(pt_->buffer());
-
-	    sprintf(pt_->buffer() + offset, 
-		    "-Ma %x -Md %x -Ms %x -Mt %x ",
-		    mh->dh_duration,
-		    ETHER_ADDR(mh->dh_da),
-		    ETHER_ADDR(mh->dh_sa),
-		    GET_ETHER_TYPE(mh->dh_body));
-		    
-	     return;
+	    if (strcmp(mactype, "Mac/SMAC") == 0) {
+		    format_smac(p, offset);
+	    } else {
+		    format_mac(p, offset);
+	    }
+	    return;
 	}
 
 
 #ifdef LOG_POSITION
-        double x = 0.0, y = 0.0, z = 0.0;
+        x = 0.0, y = 0.0, z = 0.0;
         node_->getLoc(&x, &y, &z);
 #endif
 
 	sprintf(pt_->buffer() + offset,
 #ifdef LOG_POSITION
-		"%c %.9f %d (%6.2f %6.2f) %3s %4s %d %s %d [%x %x %x %x] ",
+		"%c %.9f %d (%6.2f %6.2f) %3s %4s %d %s %d ",
 #else
-		"%c %.9f _%d_ %3s %4s %d %s %d [%x %x %x %x] ",
+		"%c %.9f _%d_ %3s %4s %d %s %d",
 #endif
 		op,
 		Scheduler::instance().clock(),
@@ -234,28 +253,113 @@ CMUTrace::format_mac(Packet *p, const char *why, int offset)
 		  (mh->dh_fc.fc_subtype == MAC_Subtype_RTS) ? "RTS"  :
 		  (mh->dh_fc.fc_subtype == MAC_Subtype_CTS) ? "CTS"  :
 		  (mh->dh_fc.fc_subtype == MAC_Subtype_ACK) ? "ACK"  :
-		  "UNKN"
-		  ) : packet_info.name(ch->ptype())),
-		
-		ch->size(),
-
-		//*((u_int16_t*) &mh->dh_fc),
-		mh->dh_duration,
-		ETHER_ADDR(mh->dh_da),
-		ETHER_ADDR(mh->dh_sa),
-		GET_ETHER_TYPE(mh->dh_body));
+		  //<zheng: add for 802.15.4>
+		  (mh->dh_fc.fc_subtype == MAC_Subtype_Beacon) ? "BCN"  :		//Beacon
+		  (mh->dh_fc.fc_subtype == MAC_Subtype_Command_AssoReq) ? "CM1"  :	//CMD: Association request
+		  (mh->dh_fc.fc_subtype == MAC_Subtype_Command_AssoRsp) ? "CM2"  :	//CMD: Association response
+		  (mh->dh_fc.fc_subtype == MAC_Subtype_Command_DAssNtf) ? "CM3"  :	//CMD: Disassociation notification
+		  (mh->dh_fc.fc_subtype == MAC_Subtype_Command_DataReq) ? "CM4"  :	//CMD: Data request
+		  (mh->dh_fc.fc_subtype == MAC_Subtype_Command_PIDCNtf) ? "CM5"  :	//CMD: PAN ID conflict notification
+		  (mh->dh_fc.fc_subtype == MAC_Subtype_Command_OrphNtf) ? "CM6"  :	//CMD: Orphan notification
+		  (mh->dh_fc.fc_subtype == MAC_Subtype_Command_BconReq) ? "CM7"  :	//CMD: Beacon request
+		  (mh->dh_fc.fc_subtype == MAC_Subtype_Command_CoorRea) ? "CM8"  :	//CMD: Coordinator realignment
+		  (mh->dh_fc.fc_subtype == MAC_Subtype_Command_GTSReq) ? "CM9"  :	//CMD: GTS request
+		  //</zheng: add for 802.15.4>
+		  "UNKN") :
+		 (ch->ptype() == PT_SMAC) ? (
+		  (sh->type == RTS_PKT) ? "RTS" :
+		  (sh->type == CTS_PKT) ? "CTS" :
+		  (sh->type == ACK_PKT) ? "ACK" :
+		  (sh->type == SYNC_PKT) ? "SYNC" :
+		  "UNKN") : 
+		 packet_info.name(ch->ptype())),
+		ch->size());
+	
+	offset = strlen(pt_->buffer());
 
+	if (strncmp (mactype, "Mac/SMAC", 8) == 0) {
+		format_smac(p, offset);
+	} else {
+		format_mac(p, offset);
+        }
+	
 	offset = strlen(pt_->buffer());
 
 	if (thisnode) {
 		if (thisnode->energy_model()) {
+			// log detailed energy consumption
+			// total energy and breakdown in idle, sleep, transmit and receive modes
 			sprintf(pt_->buffer() + offset,
-				"[energy %f] ",
-				thisnode->energy_model()->energy());
+				"[energy %f ei %.3f es %.3f et %.3f er %.3f] ",
+				thisnode->energy_model()->energy(),
+				thisnode->energy_model()->ei(),
+				thisnode->energy_model()->es(),				
+				thisnode->energy_model()->et(),
+				thisnode->energy_model()->er());				
 		}
         }
 }
 
+void
+CMUTrace::format_mac(Packet *p, int offset)
+{
+	struct hdr_mac802_11 *mh = HDR_MAC802_11(p);
+	
+	if (pt_->tagged()) {
+		sprintf(pt_->buffer() + offset,
+			"-M:dur %x -M:s %x -M:d %x -M:t %x ",
+			mh->dh_duration,		// MAC: duration
+			
+			// change wrt Mike's code
+			//ETHER_ADDR(mh->dh_da),		// MAC: source
+			//ETHER_ADDR(mh->dh_sa),		// MAC: destination
+			ETHER_ADDR(mh->dh_ra),          // MAC: source
+                       ETHER_ADDR(mh->dh_ta),          // MAC: destination
+
+
+			GET_ETHER_TYPE(mh->dh_body));	// MAC: type
+	} else if (newtrace_) {
+		sprintf(pt_->buffer() + offset, 
+			"-Ma %x -Md %x -Ms %x -Mt %x ",
+			mh->dh_duration,
+			
+			// change wrt Mike's code
+			//ETHER_ADDR(mh->dh_da),
+			//ETHER_ADDR(mh->dh_sa),
+
+	   		ETHER_ADDR(mh->dh_ra),
+	                   ETHER_ADDR(mh->dh_ta),
+
+			GET_ETHER_TYPE(mh->dh_body));
+	} else {
+		sprintf(pt_->buffer() + offset,
+			" [%x %x %x %x] ",
+			//*((u_int16_t*) &mh->dh_fc),
+			mh->dh_duration,
+			
+			// change wrt Mike's code
+			//ETHER_ADDR(mh->dh_da),
+			//ETHER_ADDR(mh->dh_sa),
+			ETHER_ADDR(mh->dh_ra),
+                        ETHER_ADDR(mh->dh_ta),
+
+
+			GET_ETHER_TYPE(mh->dh_body));
+	}
+}
+
+void
+CMUTrace::format_smac(Packet *p, int offset)
+{
+	struct hdr_smac *sh = HDR_SMAC(p);
+	sprintf(pt_->buffer() + offset,
+		" [%.2f %d %d] ",
+		sh->duration,
+		sh->dstAddr,
+		sh->srcAddr);
+}
+	
+
 void
 CMUTrace::format_ip(Packet *p, int offset)
 {
@@ -301,6 +405,8 @@ CMUTrace::format_ip(Packet *p, int offset)
 	}
 }
 
+// Note:  HDLC format (format_hdlc()) has moved to satellite tracing
+
 void
 CMUTrace::format_arp(Packet *p, int offset)
 {
@@ -438,6 +544,79 @@ CMUTrace::format_tcp(Packet *p, int offset)
 	}
 }
 
+/* Armando L. Caro Jr. <acaro@@cis,udel,edu> 6/5/2002
+ * (with help from Florina Almenrez <florina@@it,uc3m,es>)
+ */
+void
+CMUTrace::format_sctp(Packet* p,int offset)
+{
+	struct hdr_cmn *ch = HDR_CMN(p);
+	struct hdr_sctp *sh = HDR_SCTP(p);
+	//struct hdr_ip *ih = HDR_IP(p);
+	char cChunkType;
+  
+	for(u_int i = 0; i < sh->NumChunks(); i++) {
+		switch(sh->SctpTrace()[i].eType) {
+		case SCTP_CHUNK_INIT:
+		case SCTP_CHUNK_INIT_ACK:
+		case SCTP_CHUNK_COOKIE_ECHO:
+		case SCTP_CHUNK_COOKIE_ACK:
+			cChunkType = 'I';       // connection initialization
+			break;
+      
+		case SCTP_CHUNK_DATA:
+			cChunkType = 'D';
+			break;
+      
+		case SCTP_CHUNK_SACK:
+			cChunkType = 'S';
+			break;
+      
+		case SCTP_CHUNK_FORWARD_TSN:
+			cChunkType = 'R';
+			break;
+
+		case SCTP_CHUNK_HB:
+			cChunkType = 'H';
+			break;
+			
+		case SCTP_CHUNK_HB_ACK:
+			cChunkType = 'B';
+			break;
+		default:
+			// quiet compiler
+			cChunkType = ' ';
+			assert (false);
+			break;
+		}
+    
+		if( newtrace_ ) {
+			sprintf(pt_->buffer() + offset,
+				"-Pn sctp -Pnc %d -Pct %c "
+				"-Ptsn %d -Psid %d -Pssn %d "
+				"-Pf %d -Po %d ",
+				sh->NumChunks(),
+				cChunkType,
+				sh->SctpTrace()[i].uiTsn,
+				sh->SctpTrace()[i].usStreamId,
+				sh->SctpTrace()[i].usStreamSeqNum,
+				ch->num_forwards(),
+				ch->opt_num_forwards());
+		}
+		else {
+			sprintf(pt_->buffer() + offset,
+				"[%d %c %d %d %d] %d %d",
+				sh->NumChunks(),
+				cChunkType,
+				sh->SctpTrace()[i].uiTsn,
+				sh->SctpTrace()[i].usStreamId,
+				sh->SctpTrace()[i].usStreamSeqNum,
+				ch->num_forwards(),
+				ch->opt_num_forwards());
+		}
+	}
+}
+
 void
 CMUTrace::format_rtp(Packet *p, int offset)
 {
@@ -719,6 +898,38 @@ CMUTrace::nam_format(Packet *p, int offset)
 	char colors[32];
 	int next_hop = -1 ;
 
+// change wrt Mike's code
+	assert(type_ != EOT);
+
+
+
+	//<zheng: add for 802.15.4>
+
+	//Actually we only need to handle MAC layer for nam (but should display dropping for other layers)
+	//if (strcmp(tracename,"MAC") != 0)
+	//if ((op != 'D')&&(op != 'd'))
+	//	return;
+
+	struct hdr_mac802_11 *mh = HDR_MAC802_11(p);
+	char ptype[11];
+	strcpy(ptype,
+	((ch->ptype() == PT_MAC) ? (
+	  (mh->dh_fc.fc_subtype == MAC_Subtype_RTS) ? "RTS"  :
+	  (mh->dh_fc.fc_subtype == MAC_Subtype_CTS) ? "CTS"  :
+	  (mh->dh_fc.fc_subtype == MAC_Subtype_ACK) ? "ACK"  :
+	  (mh->dh_fc.fc_subtype == MAC_Subtype_Beacon) ? "BCN"  :		//Beacon
+	  (mh->dh_fc.fc_subtype == MAC_Subtype_Command_AssoReq) ? "CM1"  :	//CMD: Association request
+	  (mh->dh_fc.fc_subtype == MAC_Subtype_Command_AssoRsp) ? "CM2"  :	//CMD: Association response
+	  (mh->dh_fc.fc_subtype == MAC_Subtype_Command_DAssNtf) ? "CM3"  :	//CMD: Disassociation notification
+	  (mh->dh_fc.fc_subtype == MAC_Subtype_Command_DataReq) ? "CM4"  :	//CMD: Data request
+	  (mh->dh_fc.fc_subtype == MAC_Subtype_Command_PIDCNtf) ? "CM5"  :	//CMD: PAN ID conflict notification
+	  (mh->dh_fc.fc_subtype == MAC_Subtype_Command_OrphNtf) ? "CM6"  :	//CMD: Orphan notification
+	  (mh->dh_fc.fc_subtype == MAC_Subtype_Command_BconReq) ? "CM7"  :	//CMD: Beacon request
+	  (mh->dh_fc.fc_subtype == MAC_Subtype_Command_CoorRea) ? "CM8"  :	//CMD: Coordinator realignment
+	  (mh->dh_fc.fc_subtype == MAC_Subtype_Command_GTSReq) ? "CM9"  :	//CMD: GTS request
+	  "UNKN"
+	  ) : packet_info.name(ch->ptype())));
+	//</zheng: add for 802.15.4>
         int dst = Address::instance().get_nodeaddr(ih->daddr());
 
 	nextnode = Node::get_node_by_address(ch->next_hop_);
@@ -776,36 +987,64 @@ CMUTrace::nam_format(Packet *p, int offset)
 			pkt_color = 1;
 		}
 	}
+
+	//<zheng: add for 802.15.4>
+	if (Nam802_15_4::Nam_Status)
+	{
+		ATTRIBUTELINK *attr;
+		int t_src,t_dst;
+		if (ch->ptype() == PT_MAC)
+		{
+			t_src = p802_15_4macSA(p);
+			t_dst = p802_15_4macDA(p);;
+		}
+		else
+		{
+			t_src = HDR_IP(p)->saddr();
+			t_dst = HDR_IP(p)->daddr();
+		}
+		attr = findAttrLink(HDR_CMN(p)->ptype(),t_src,t_dst);
+		if (attr == NULL)
+			attr = findAttrLink(HDR_CMN(p)->ptype());
+		if (attr != NULL)
+			HDR_LRWPAN(p)->attribute = attr->attribute;
+		else
+			HDR_LRWPAN(p)->attribute = 0;
+		if (HDR_LRWPAN(p)->attribute >= 32)
+			pkt_color = HDR_LRWPAN(p)->attribute;
+	}
+	//</zheng: add for 802.15.4>
+
 	// convert to nam format 
 	if (op == 's') op = 'h' ;
 	if (op == 'D') op = 'd' ;
 	if (op == 'h') {
-	   sprintf(pt_->nbuffer(),
-		"+ -t %.9f -s %d -d %d -p %s -e %d -c 2 -a %d -i %d -k %3s ",
-		Scheduler::instance().clock(),
-		src_,                           // this node
-		next_hop,
-		packet_info.name(ch->ptype()),
-		ch->size(),
-		pkt_color,   
-		ch->uid(),
-		tracename);
+		sprintf(pt_->nbuffer(),
+			"+ -t %.9f -s %d -d %d -p %s -e %d -c 2 -a %d -i %d -k %3s ",
+			Scheduler::instance().clock(),
+			src_,                           // this node
+			next_hop,
+			ptype,			//<zheng: modify for 802.15.4>packet_info.name(ch->ptype()),
+			ch->size(),
+			pkt_color,   
+			ch->uid(),
+			tracename);
 
-	   offset = strlen(pt_->nbuffer());
-	   pt_->namdump();
-	   sprintf(pt_->nbuffer() ,
-		"- -t %.9f -s %d -d %d -p %s -e %d -c 2 -a %d -i %d -k %3s",
-		Scheduler::instance().clock(),
-		src_,                           // this node
-		next_hop,
-		packet_info.name(ch->ptype()),
-		ch->size(),
-		pkt_color,
-		ch->uid(),
-		tracename);
-
-	   offset = strlen(pt_->nbuffer());
-           pt_->namdump();
+		offset = strlen(pt_->nbuffer());
+		pt_->namdump();
+		sprintf(pt_->nbuffer() ,
+			"- -t %.9f -s %d -d %d -p %s -e %d -c 2 -a %d -i %d -k %3s",
+			Scheduler::instance().clock(),
+			src_,                           // this node
+			next_hop,
+			ptype,			//<zheng: modify for 802.15.4>packet_info.name(ch->ptype()),
+			ch->size(),
+			pkt_color,
+			ch->uid(),
+			tracename);
+		
+		offset = strlen(pt_->nbuffer());
+		pt_->namdump();
 	}
 
         // if nodes are too far from each other
@@ -820,6 +1059,11 @@ CMUTrace::nam_format(Packet *p, int offset)
 	if(type_ == RECV && dst == -1 )dst = src_ ; //broadcasting event
 
         if (energy != -1) { //energy model being turned on
+	   if (src_ >= MAX_NODE) {
+		   fprintf (stderr, "node id must be < %d\n",
+			    MAX_NODE);
+	       exit(0);
+	   }
 	   if (nodeColor[src_] != energyLevel ) { //only dump it when node  
 	       sprintf(pt_->nbuffer() ,                    //color change
 	          "n -t %.9f -s %d -S COLOR %s",
@@ -832,24 +1076,23 @@ CMUTrace::nam_format(Packet *p, int offset)
 	    }   
         }
 
-
-	// don't generate an "r" event for a broadcast packet
-	// as it was already generated when the "h" event was written
-	if (next_hop == -1 && op == 'r')
-		return;
-	
-	sprintf(pt_->nbuffer(),
+	sprintf(pt_->nbuffer() ,
 		"%c -t %.9f -s %d -d %d -p %s -e %d -c 2 -a %d -i %d -k %3s",
 		op,
 		Scheduler::instance().clock(),
 		src_,                           // this node
 		next_hop,
-		packet_info.name(ch->ptype()),
+		ptype,			//<zheng: modify for 802.15.4>packet_info.name(ch->ptype()),
 		ch->size(),
 		pkt_color,
 		ch->uid(),
 		tracename);
 
+//<zheng: ns 2.27 removed the following part, but we need it to control the broadcast radius>
+if (Nam802_15_4::Nam_Status)
+{
+	if ((strcmp(tracename, "AGT") != 0) || ((u_int32_t)(ih->daddr()) == IP_BROADCAST))		//<zheng: add: next_hop info not available at agent level>
+											//(doesn't really matter -- seems agent level has no effect on nam)
 	if (next_hop == -1 && op == 'h') {
 		// print extra fields for broadcast packets
 
@@ -863,28 +1106,18 @@ CMUTrace::nam_format(Packet *p, int offset)
 		// duration is calculated based on the radius and
 		// the speed of light (299792458 m/s)
 		double duration = (bradius/299792458.0)*duration_scaling_factor_;
-
-
+		//<zheng: add -- the duration in 802.15.4 could be very small and rounded to 0.0>
+		if (Nam802_15_4::Nam_Status)
+		if (duration < 0.000000001)
+			duration = 0.000000001;
+		//</zheng: add>
 		sprintf(pt_->nbuffer() + strlen(pt_->nbuffer()),
 			" -R %.2f -D %.2f",
 			radius,
 			duration);
-
-		// schedule "r" event
-		Tcl& tcl = Tcl::instance();
-		tcl.evalf("[Simulator instance] at %f {[Simulator instance] puts-nam-traceall {r -t %.9f -s %d -d %d -p %s -e %d -c 2 -a %d -i %d -k %3s -R %.2f -D %.2f}}",
-			Scheduler::instance().clock() + duration,
-			Scheduler::instance().clock() + duration,
-			src_,
-			next_hop,
-			packet_info.name(ch->ptype()),
-			ch->size(),
-			pkt_color,
-			ch->uid(),
-			tracename,
-			radius,
-			duration);
 	}
+}
+//</zheng>
 
 	offset = strlen(pt_->nbuffer());
 	pt_->namdump();
@@ -898,13 +1131,14 @@ void CMUTrace::format(Packet* p, const char *why)
 	/*
 	 * Log the MAC Header
 	 */
-	format_mac(p, why, offset);
+	format_mac_common(p, why, offset);
 
 	if (pt_->namchannel()) 
 		nam_format(p, offset);
 	offset = strlen(pt_->buffer());
 	switch(ch->ptype()) {
 	case PT_MAC:
+	case PT_SMAC:
 		break;
 	case PT_ARP:
 		format_arp(p, offset);
@@ -933,6 +1167,11 @@ void CMUTrace::format(Packet* p, const char *why)
 		case PT_ACK:
 			format_tcp(p, offset);
 			break;
+		case PT_SCTP:
+			/* Armando L. Caro Jr. <acaro@@cis,udel,edu> 6/5/2002
+			 */
+			format_sctp(p, offset);
+			break;
 		case PT_CBR:
 			format_rtp(p, offset);
 			break;
@@ -942,9 +1181,12 @@ void CMUTrace::format(Packet* p, const char *why)
 		case PT_PING:
 			break;
 		default:
+		/*<zheng: del -- there are many more new packet types added, like PT_EXP (poisson traffic belongs to this type)>
 			fprintf(stderr, "%s - invalid packet type (%s).\n",
 				__PRETTY_FUNCTION__, packet_info.name(ch->ptype()));
 			exit(1);
+		</zheng: del>*/
+			break;		//zheng: add
 		}
 	}
 }
@@ -1040,6 +1282,7 @@ int CMUTrace::node_energy()
 	return 0;
 }
 
+//<zheng: ns 2.27 removed the following part, but we need it to control the broadcast radius>
 void CMUTrace::calculate_broadcast_parameters() {
 	// Calculate the maximum distance at which a packet can be received
 	// based on the two-ray reflection model using the current default
@@ -1061,6 +1304,17 @@ void CMUTrace::calculate_broadcast_parameters() {
 	tcl.evalc("Antenna/OmniAntenna set Z_");
 	h = atof(tcl.result());
 	bradius = pow(P_t*G_r*G_t*pow(h,4.0)/(P_r*L), 0.25);
+	//<zheng: add for 802.15.4>
+	//the above calculation is not accurate for short distance
+	double PI,freq,lambda,crossover_dist;
+	PI = 3.14159265359;
+	tcl.evalc("Phy/WirelessPhy set freq_");
+	freq = atof(tcl.result());
+	lambda = 3.0e8/freq;
+	crossover_dist = (4 * PI * h * h) / lambda;
+	if (bradius < crossover_dist)	//need re-calculation
+		bradius = pow(P_t * G_r * G_t * pow(lambda, 2.0)/(P_r * L), 0.5)/(4 * PI);
+	//</zheng: add for 802.15.4>
 
 	// Also get the scaling factors
 	tcl.evalc("CMUTrace set radius_scaling_factor_");
@@ -1068,5 +1322,4 @@ void CMUTrace::calculate_broadcast_parameters() {
 	tcl.evalc("CMUTrace set duration_scaling_factor_");
 	duration_scaling_factor_ = atof(tcl.result());
 }
-
-
+//</zheng>
